from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from je_editor.pyside_ui.code.plaintext_code_edit.code_edit_plaintext import CodeEditor

from PySide6.QtCore import QRegularExpression
from PySide6.QtGui import QSyntaxHighlighter
from PySide6.QtGui import QTextCharFormat

from je_editor.pyside_ui.code.syntax.syntax_setting import syntax_word_setting_dict, syntax_rule_setting_dict, \
    syntax_extend_setting_dict
from je_editor.utils.logging.loggin_instance import jeditor_logger


class PythonHighlighter(QSyntaxHighlighter):
    def __init__(self, parent=None, main_window: CodeEditor = None):
        jeditor_logger.info(f"Init PythonHighlighter parent: {parent}")
        super().__init__(parent)

        self.highlight_rules = []
        if main_window.current_file is not None:
            current_file_suffix = Path(main_window.current_file).suffix
        else:
            current_file_suffix = ".py"
        if current_file_suffix == ".py":
            # Highlight
            for rule_variable_dict in syntax_word_setting_dict.values():
                color = rule_variable_dict.get("color")
                text_char_format = QTextCharFormat()
                text_char_format.setForeground(color)
                for word in rule_variable_dict.get("words"):
                    pattern = QRegularExpression(rf"\b{word}\b")
                    self.highlight_rules.append((pattern, text_char_format))
            for rule_variable_dict in syntax_rule_setting_dict.values():
                color = rule_variable_dict.get("color")
                text_char_format = QTextCharFormat()
                text_char_format.setForeground(color)
                for rule in rule_variable_dict.get("rules"):
                    pattern = QRegularExpression(rule)
                    self.highlight_rules.append((pattern, text_char_format))
        else:
            if syntax_extend_setting_dict.get(current_file_suffix):
                for rule_variable_dict in syntax_extend_setting_dict.get(current_file_suffix).values():
                    color = rule_variable_dict.get("color")
                    text_char_format = QTextCharFormat()
                    text_char_format.setForeground(color)
                    for word in rule_variable_dict.get("words"):
                        pattern = QRegularExpression(rf"\b{word}\b")
                        self.highlight_rules.append((pattern, text_char_format))
            else:
                pass

    def highlightBlock(self, text) -> None:
        jeditor_logger.info(f"PythonHighlighter highlightBlock text: {text}")
        for pattern, pattern_format in self.highlight_rules:
            match_iterator = pattern.globalMatch(text)
            while match_iterator.hasNext():
                match = match_iterator.next()
                self.setFormat(match.capturedStart(), match.capturedLength(), pattern_format)



from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING

from je_editor.utils.logging.loggin_instance import jeditor_logger

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.editor.editor_widget import EditorWidget
    from je_editor.pyside_ui.main_ui.editor.editor_widget_dock import FullEditorWidget

from typing import Union, List

import jedi
from PySide6 import QtGui
from PySide6.QtCore import Qt, QRect
from PySide6.QtGui import QPainter, QTextCharFormat, QTextFormat, QKeyEvent, QAction, QTextDocument, QTextCursor, \
    QTextOption
from PySide6.QtWidgets import QPlainTextEdit, QWidget, QTextEdit, QCompleter
from jedi.api.classes import Completion

from je_editor.pyside_ui.code.syntax.python_syntax import PythonHighlighter
from je_editor.pyside_ui.dialog.search_ui.search_text_box import SearchBox
from je_editor.pyside_ui.main_ui.save_settings.user_color_setting_file import actually_color_dict


def venv_check():
    jeditor_logger.info("code_edit_plaintext.py venv check")
    venv_path = Path(str(Path.cwd()) + "/venv")
    return venv_path


class CodeEditor(QPlainTextEdit):
    """
    Extend QPlainTextEdit,
    Add line, edit tab distance, add highlighter, add search text
    """

    def __init__(self, main_window: Union[EditorWidget, FullEditorWidget]):
        jeditor_logger.info(f"Init CodeEditor main_window: {main_window}")
        super().__init__()
        # Jedi
        self.env = None
        self.check_env()
        # Self main window (parent)
        self.main_window = main_window
        self.current_file = main_window.current_file

        self.skip_popup_behavior_list = [
            Qt.Key.Key_Enter, Qt.Key.Key_Return, Qt.Key.Key_Up, Qt.Key.Key_Down,
            Qt.Key.Key_Tab, Qt.Key.Key_Backtab, Qt.Key.Key_Space, Qt.Key.Key_Backspace
        ]
        self.need_complete_list = [
            Qt.Key.Key_A, Qt.Key.Key_B, Qt.Key.Key_C, Qt.Key.Key_D, Qt.Key.Key_E, Qt.Key.Key_F,
            Qt.Key.Key_G, Qt.Key.Key_H, Qt.Key.Key_I, Qt.Key.Key_J, Qt.Key.Key_K, Qt.Key.Key_L,
            Qt.Key.Key_M, Qt.Key.Key_N, Qt.Key.Key_O, Qt.Key.Key_P, Qt.Key.Key_Q, Qt.Key.Key_R,
            Qt.Key.Key_S, Qt.Key.Key_T, Qt.Key.Key_U, Qt.Key.Key_V, Qt.Key.Key_W, Qt.Key.Key_X,
            Qt.Key.Key_Y, Qt.Key.Key_Z
        ]
        self.search_box = None
        self.line_number: LineNumber = LineNumber(self)
        self.blockCountChanged.connect(self.update_line_number_area_width)
        self.updateRequest.connect(self.update_line_number_area)
        self.update_line_number_area_width(0)
        self.textChanged.connect(self.highlight_current_line)
        self.setTabStopDistance(
            QtGui.QFontMetricsF(self.font()).horizontalAdvance("        ")
        )
        self.highlighter = PythonHighlighter(self.document(), main_window=self)
        self.highlight_current_line()
        self.setLineWrapMode(self.LineWrapMode.NoWrap)
        self.setWordWrapMode(QTextOption.WrapMode.WrapAnywhere)
        # Search Text
        self.search_action = QAction("Search")
        self.search_action.setShortcut("Ctrl+f")
        self.search_action.triggered.connect(
            self.start_search_dialog
        )
        # Add actions
        self.addAction(self.search_action)
        # Complete
        self.completer: Union[None, QCompleter] = None
        self.set_complete([])

    def reset_highlighter(self):
        jeditor_logger.info("CodeEditor reset_highlighter")
        self.highlighter = PythonHighlighter(self.document(), main_window=self)
        self.highlight_current_line()

    def check_env(self):
        jeditor_logger.info("CodeEditor check_env")
        path = venv_check()
        if path.exists():
            self.env = jedi.create_environment(str(path))

    def set_complete(self, list_to_complete: list) -> None:
        """
        Set complete and bind.
        :param list_to_complete: keyword list to complete.
        :return: None
        """
        jeditor_logger.info(f"CodeEditor set_complete list_to_complete: {list_to_complete}")
        completer = QCompleter(list_to_complete)
        completer.activated.connect(self.insert_completion)
        completer.setWidget(self)
        completer.setWrapAround(False)
        completer.setCompletionMode(QCompleter.CompletionMode.PopupCompletion)
        completer.setCaseSensitivity(Qt.CaseSensitivity.CaseInsensitive)
        self.completer = completer

    def insert_completion(self, completion) -> None:
        """
        insert complete keyword to editor.
        :param completion: completion text
        :return: None
        """
        jeditor_logger.info(f"CodeEditor insert_completion completion: {completion}")
        if self.completer.widget() != self:
            return
        text_cursor = self.textCursor()
        extra = len(completion) - len(self.completer.completionPrefix())
        text_cursor.movePosition(QTextCursor.MoveOperation.Left)
        text_cursor.movePosition(QTextCursor.MoveOperation.EndOfWord)
        text_cursor.insertText(completion[-extra:])
        self.setTextCursor(text_cursor)

    @property
    def text_under_cursor(self):
        jeditor_logger.info("CodeEditor text_under_cursor")
        # Find text under cursor
        text_cursor = self.textCursor()
        text_cursor.select(QTextCursor.SelectionType.WordUnderCursor)
        return text_cursor.selectedText()

    def focusInEvent(self, e) -> None:
        jeditor_logger.info(f"CodeEditor focusInEvent event: {e}")
        if self.completer:
            self.completer.setWidget(self)
        QPlainTextEdit.focusInEvent(self, e)

    def complete(self) -> None:
        """
        Keyword autocomplete
        :return:  None
        """
        jeditor_logger.info("CodeEditor complete")
        prefix = self.text_under_cursor
        if self.env is not None:
            script = jedi.Script(code=self.toPlainText(), environment=self.env)
        else:
            script = jedi.Script(code=self.toPlainText())
        jedi_complete_list: List[Completion] = script.complete(
            self.textCursor().blockNumber() + 1,
            len(self.textCursor().document().findBlockByLineNumber(self.textCursor().blockNumber()).text())
        )
        if len(jedi_complete_list) > 0:
            new_complete_list = list()
            for complete_text in jedi_complete_list:
                new_complete_list.append(complete_text.name)
            self.set_complete(new_complete_list)
        self.completer.setCompletionPrefix(prefix)
        popup = self.completer.popup()
        cursor_rect = self.cursorRect()
        popup.setCurrentIndex(self.completer.completionModel().index(0, 0))
        cursor_rect.setWidth(self.completer.popup().rect().size().width())
        self.completer.complete(cursor_rect)

    def start_search_dialog(self) -> None:
        """
        Show search box ui and bind.
        :return: None
        """
        jeditor_logger.info("CodeEditor start_search_dialog")
        # Search box connect to function
        self.search_box = SearchBox()
        self.search_box.search_back_button.clicked.connect(
            self.find_back_text
        )
        self.search_box.search_next_button.clicked.connect(
            self.find_next_text
        )
        self.search_box.show()

    def find_next_text(self) -> None:
        """
        Find next match text.
        :return: None
        """
        jeditor_logger.info("CodeEditor find_next_text")
        if self.search_box.isVisible():
            text = self.search_box.command_input.text()
            self.find(text)

    def find_back_text(self) -> None:
        """
        Find back match text.
        :return: None
        """
        jeditor_logger.info("CodeEditor find_back_text")
        if self.search_box.isVisible():
            text = self.search_box.command_input.text()
            self.find(text, QTextDocument.FindFlag.FindBackward)

    def line_number_paint(self, event) -> None:
        jeditor_logger.info(f"CodeEditor line_number_paint event: {event}")
        painter = QPainter(self.line_number)
        painter.fillRect(event.rect(), actually_color_dict.get("line_number_background_color"))
        block = self.firstVisibleBlock()
        block_number = block.blockNumber()
        top = self.blockBoundingGeometry(block).translated(self.contentOffset()).top()
        bottom = top + self.blockBoundingRect(block).height()
        while block.isValid() and top <= event.rect().bottom():
            if block.isVisible() and bottom >= event.rect().top():
                number = str(block_number + 1)
                painter.setPen(actually_color_dict.get("line_number_color"))
                painter.drawText(
                    0,
                    top,
                    self.line_number.width(),
                    self.fontMetrics().height(),
                    Qt.AlignmentFlag.AlignCenter,
                    number,
                )
            block = block.next()
            top = bottom
            bottom = top + self.blockBoundingRect(block).height()
            block_number += 1

    def line_number_width(self) -> int:
        jeditor_logger.info("CodeEditor line_number_width")
        digits = len(str(self.blockCount()))
        space = 12 * digits
        return space

    def update_line_number_area_width(self, value) -> None:
        jeditor_logger.info(f"CodeEditor update_line_number_area_width value: {value}")
        self.setViewportMargins(self.line_number_width(), 0, 0, 0)

    def resizeEvent(self, event) -> None:
        """
        Resize line number paint.
        :param event: QT event.
        :return: None
        """
        jeditor_logger.info(f"CodeEditor resizeEvent event:{event}")
        QPlainTextEdit.resizeEvent(self, event)
        cr = self.contentsRect()
        self.line_number.setGeometry(
            QRect(cr.left(), cr.top(), self.line_number_width(), cr.height()),
        )

    def update_line_number_area(self, rect, dy) -> None:
        """
        Set line number.
        :param rect: line number rect.
        :param dy: update or not.
        :return: None
        """
        jeditor_logger.info(f"CodeEditor update_line_number_area rect: {rect}, dy: {dy}")
        if dy:
            self.line_number.scroll(0, dy)
        else:
            self.line_number.update(
                0,
                rect.y(),
                self.line_number.width(),
                rect.height(),
            )
        if rect.contains(self.viewport().rect()):
            self.update_line_number_area_width(0)

    def highlight_current_line(self) -> None:
        """
        Change current line color.
        :return: None
        """
        jeditor_logger.info("CodeEditor highlight_current_line")
        selections = []
        if not self.isReadOnly():
            formats = QTextCharFormat()
            selection = QTextEdit.ExtraSelection()
            selection.format = formats
            color_of_the_line = actually_color_dict.get("current_line_color")
            selection.cursor = self.textCursor()
            selection.cursor.clearSelection()
            selections.append(selection)
            selection.format.setBackground(color_of_the_line)
            selection.format.setProperty(QTextFormat.FullWidthSelection, True)
        self.setExtraSelections(selections)

    def keyPressEvent(self, event: QKeyEvent) -> None:
        """
        Catch Soft new line (key, shift + enter)
        :param event: keypress event
        :return: None
        """
        # Catch soft wrap shift + return (line nuber not working on soft warp)
        key = event.key()
        jeditor_logger.info(f"CodeEditor keyPressEvent event: {event} key: {key}")
        if event.modifiers() and Qt.Modifier.CTRL:
            if key == Qt.Key.Key_B:
                if self.env is not None:
                    script = jedi.Script(code=self.toPlainText(), environment=self.env)
                else:
                    script = jedi.Script(code=self.toPlainText())
                goto_list: List[jedi.api.classes.Name] = script.goto(
                    self.textCursor().blockNumber() + 1, self.textCursor().positionInBlock())
                if len(goto_list) > 0:
                    path = goto_list[0].module_path
                    if path is not None and path.exists():
                        if self.main_window.current_file != str(path):
                            self.main_window.main_window.go_to_new_tab(path)
                    else:
                        self.textCursor().setPosition(goto_list[0].line - 1)
                return
        if self.completer.popup().isVisible() and key in self.skip_popup_behavior_list:
            self.completer.popup().close()
            event.ignore()
            return
        if event.modifiers() and Qt.Modifier.SHIFT:
            if key == Qt.Key.Key_Enter or key == Qt.Key.Key_Return:
                event.ignore()
                return
        super().keyPressEvent(event)
        self.highlight_current_line()
        if key in self.need_complete_list and self.completer is not None:
            if self.completer.popup().isVisible():
                self.completer.popup().close()
            self.complete()

    def mousePressEvent(self, event) -> None:
        jeditor_logger.info(f"CodeEditor mousePressEvent event: {event}")
        # Highlight mouse click line
        super().mousePressEvent(event)
        self.highlight_current_line()


class LineNumber(QWidget):
    """
    Used to paint line number.
    """

    def __init__(self, editor):
        jeditor_logger.info("Init LineNumber")
        QWidget.__init__(self, parent=editor)
        self.editor = editor

    def paintEvent(self, event) -> None:
        jeditor_logger.info(f"LineNumber paintEvent event: {event}")
        self.editor.line_number_paint(event)



from __future__ import annotations

import os
from pathlib import Path
from typing import TYPE_CHECKING

from je_editor.pyside_ui.code.auto_save.auto_save_manager import file_is_open_manager_dict
from je_editor.utils.logging.loggin_instance import jeditor_logger

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain
from PySide6.QtWidgets import QFileDialog

from je_editor.pyside_ui.main_ui.editor.editor_widget import EditorWidget
from je_editor.utils.file.save.save_file import write_file


def choose_file_get_save_file_path(parent_qt_instance: EditorMain) -> bool:
    """
    :param parent_qt_instance: Pyside parent
    :return: save code edit content to file
    """
    jeditor_logger.info("save_file_dialog.py choose_file_get_save_file_path"
                        f" parent_qt_instance: {parent_qt_instance}")
    widget = parent_qt_instance.tab_widget.currentWidget()
    if isinstance(widget, EditorWidget):
        file_path = QFileDialog().getSaveFileName(
            parent=parent_qt_instance,
            dir=os.getcwd(),
            filter="""Python file (*.py);;
            HTML file (*.html);;
            File (*.*)"""
        )[0]
        if file_path is not None and file_path != "":
            widget.current_file = file_path
            write_file(file_path, widget.code_edit.toPlainText())
            path = Path(file_path)
            file_is_open_manager_dict.update({str(path): str(path.name)})
            if widget.code_save_thread is not None:
                widget.code_save_thread.file = file_path
                widget.code_save_thread.editor = widget.code_edit
            widget.rename_self_tab()
            return True
        return False



from __future__ import annotations

import os
import queue
import shlex
import subprocess
import sys
from pathlib import Path
from threading import Thread
from typing import Union, Callable

from PySide6.QtCore import QTimer
from PySide6.QtWidgets import QTextEdit

from je_editor.pyside_ui.code.running_process_manager import run_instance_manager
from je_editor.pyside_ui.main_ui.editor.editor_widget import EditorWidget
from je_editor.pyside_ui.main_ui.save_settings.user_color_setting_file import actually_color_dict
from je_editor.utils.exception.exception_tags import je_editor_init_error
from je_editor.utils.exception.exceptions import JEditorException
from je_editor.utils.logging.loggin_instance import jeditor_logger
from je_editor.utils.venv_check.check_venv import check_and_choose_venv


class ShellManager(object):

    def __init__(
            self,
            main_window: Union[EditorWidget, None] = None,
            shell_encoding: str = "utf-8",
            program_buffer: int = 1024,
            after_done_function: Union[None, Callable] = None
    ):
        jeditor_logger.info(f"Init ShellManager "
                            f"main_window: {main_window} "
                            f"shell_encoding: {shell_encoding} "
                            f"program_buffer: {program_buffer} "
                            f"after_done_function: {after_done_function}")
        """
        :param main_window: Pyside main window
        :param shell_encoding: shell command read output encoding
        :param program_buffer: buffer size
        """
        self.read_program_error_output_from_thread = None
        self.read_program_output_from_thread = None
        self.main_window: EditorWidget = main_window
        self.compiler_path = None
        self.code_result: [QTextEdit, None] = None
        self.timer: [QTimer, None] = None
        self.still_run_shell: bool = True
        self.process = None
        self.run_output_queue: queue = queue.Queue()
        self.run_error_queue: queue = queue.Queue()
        self.program_encoding: str = shell_encoding
        self.program_buffer: int = program_buffer
        self.after_done_function = after_done_function
        self.renew_path()
        run_instance_manager.instance_list.append(self)

    def renew_path(self) -> None:
        jeditor_logger.info("ShellManager renew_path")
        if self.main_window.python_compiler is None:
            # Renew compiler path
            if sys.platform in ["win32", "cygwin", "msys"]:
                venv_path = Path(os.getcwd() + "/venv/Scripts")
            else:
                venv_path = Path(os.getcwd() + "/venv/bin")
            self.compiler_path = check_and_choose_venv(venv_path)
        else:
            self.compiler_path = self.main_window.python_compiler

    def later_init(self) -> None:
        jeditor_logger.info("ShellManager later_init")
        if self.main_window is not None:
            self.code_result: QTextEdit = self.main_window.code_result
        else:
            raise JEditorException(je_editor_init_error)

    def exec_shell(self, shell_command: [str, list]) -> None:
        """
        :param shell_command: shell command will run
        :return: if error return result and True else return result and False
        """
        jeditor_logger.info(f"ShellManager exec_shell, shell_command: {shell_command}")
        try:
            self.exit_program()
            self.code_result.setTextColor(actually_color_dict.get("normal_output_color"))
            self.code_result.setPlainText("")
            if sys.platform in ["win32", "cygwin", "msys"]:
                args = shell_command
            else:
                args = shlex.split(shell_command)
            self.code_result.append(str(args))
            self.process = subprocess.Popen(
                args=args,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                stdin=subprocess.PIPE,
                shell=True,
            )
            self.still_run_shell = True
            # program output message queue thread
            self.read_program_output_from_thread = Thread(
                target=self.read_program_output_from_process,
                daemon=True
            )
            self.read_program_output_from_thread.start()
            # program error message queue thread
            self.read_program_error_output_from_thread = Thread(
                target=self.read_program_error_output_from_process,
                daemon=True
            )
            self.read_program_error_output_from_thread.start()
            # start timer
            self.timer = QTimer(self.main_window)
            self.timer.setInterval(100)
            self.timer.timeout.connect(self.pull_text)
            self.timer.start()
        except Exception as error:
            self.code_result.setTextColor(actually_color_dict.get("error_output_color"))
            self.code_result.append(str(error))
            self.code_result.setTextColor(actually_color_dict.get("normal_output_color"))
            if self.process is not None:
                self.process.terminate()

    # tkinter_ui update method
    def pull_text(self) -> None:
        jeditor_logger.info("ShellManager pull_text")
        try:
            self.code_result.setTextColor(actually_color_dict.get("normal_output_color"))
            if not self.run_output_queue.empty():
                output_message = self.run_output_queue.get_nowait()
                output_message = str(output_message).strip()
                if output_message:
                    self.code_result.append(output_message)
            self.code_result.setTextColor(actually_color_dict.get("error_output_color"))
            if not self.run_error_queue.empty():
                error_message = self.run_error_queue.get_nowait()
                error_message = str(error_message).strip()
                if error_message:
                    self.code_result.append(error_message)
            self.code_result.setTextColor(actually_color_dict.get("normal_output_color"))
        except queue.Empty:
            pass
        if self.process.returncode == 0:
            self.process_run_over()
        elif self.process.returncode is not None:
            self.process_run_over()
        if self.still_run_shell:
            # poll return code
            self.process.poll()

    def process_run_over(self):
        jeditor_logger.info("ShellManager process_run_over")
        self.timer.stop()
        self.exit_program()
        self.main_window.exec_shell = None
        if self.after_done_function is not None:
            self.after_done_function()

    # exit program change run flag to false and clean read thread and queue and process
    def exit_program(self) -> None:
        jeditor_logger.info("ShellManager exit_program")
        self.still_run_shell = False
        if self.read_program_output_from_thread is not None:
            self.read_program_output_from_thread = None
        if self.read_program_error_output_from_thread is not None:
            self.read_program_error_output_from_thread = None
        self.print_and_clear_queue()
        if self.process is not None:
            self.process.terminate()
            self.code_result.append(f"Shell command exit with code {self.process.returncode}")
            self.process = None

    def print_and_clear_queue(self) -> None:
        jeditor_logger.info("ShellManager print_and_clear_queue")
        self.run_output_queue = queue.Queue()
        self.run_error_queue = queue.Queue()

    def read_program_output_from_process(self) -> None:
        jeditor_logger.info("ShellManager read_program_output_from_process")
        while self.still_run_shell:
            program_output_data = self.process.stdout.read(
                self.program_buffer) \
                .decode(self.program_encoding, "replace")
            if self.process:
                self.process.stdout.flush()
            self.run_output_queue.put_nowait(program_output_data)

    def read_program_error_output_from_process(self) -> None:
        jeditor_logger.info("ShellManager read_program_error_output_from_process")
        while self.still_run_shell:
            program_error_output_data = self.process.stderr.read(
                self.program_buffer) \
                .decode(self.program_encoding, "replace")
            if self.process:
                self.process.stderr.flush()
            self.run_error_queue.put_nowait(program_error_output_data)



import shutil
from pathlib import Path

from je_editor.utils.exception.exception_tags import compiler_not_found_error
from je_editor.utils.exception.exceptions import JEditorExecException
from je_editor.utils.logging.loggin_instance import jeditor_logger


def check_and_choose_venv(venv_path: Path) -> str:
    jeditor_logger.info(f"check_venv.py check_and_choose_venv venv_path: {venv_path}")
    compiler_path = None
    if venv_path.is_dir() and venv_path.exists():
        compiler_path = shutil.which(
            cmd="python3",
            path=str(venv_path)
        )
    if compiler_path is None:
        compiler_path = shutil.which(
            cmd="python",
            path=str(venv_path)
        )
    else:
        compiler_path = shutil.which(cmd="python")
    if compiler_path is None:
        compiler_path = shutil.which(cmd="python")
    if compiler_path is None:
        raise JEditorExecException(compiler_not_found_error)
    return compiler_path



import os
import pathlib
import sys
from pathlib import Path
from typing import Dict, Type

import jedi.settings
from PySide6.QtCore import QTimer, QEvent
from PySide6.QtGui import QFontDatabase, QIcon, Qt
from PySide6.QtWidgets import QMainWindow, QWidget, QTabWidget
from frontengine import FrontEngineMainUI
from qt_material import QtStyleTools

from je_editor.pyside_ui.browser.browser_widget import BrowserWidget
from je_editor.pyside_ui.code.auto_save.auto_save_manager import init_new_auto_save_thread, file_is_open_manager_dict
from je_editor.pyside_ui.main_ui.editor.editor_widget import EditorWidget
from je_editor.pyside_ui.main_ui.menu.set_menu_bar import set_menu_bar
from je_editor.pyside_ui.main_ui.save_settings.user_color_setting_file import write_user_color_setting, \
    read_user_color_setting, update_actually_color_dict
from je_editor.pyside_ui.main_ui.save_settings.user_setting_file import user_setting_dict, read_user_setting, \
    write_user_setting
from je_editor.pyside_ui.main_ui.system_tray.extend_system_tray import ExtendSystemTray
from je_editor.utils.file.open.open_file import read_file
from je_editor.utils.logging.loggin_instance import jeditor_logger
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper
from je_editor.utils.redirect_manager.redirect_manager_class import redirect_manager_instance

EDITOR_EXTEND_TAB: Dict[str, Type[QWidget]] = {}


class EditorMain(QMainWindow, QtStyleTools):

    def __init__(self, debug_mode: bool = False, show_system_tray_ray: bool = False):
        jeditor_logger.info(f"Init EditorMain "
                            f"debug_mode: {debug_mode} "
                            f"show_system_tray_ray: {show_system_tray_ray}")
        super(EditorMain, self).__init__()
        # Init variable
        self.file_menu = None
        self.code_result = None
        self.code_edit = None
        self.menu = None
        self.encoding_menu = None
        self.font_size_menu = None
        self.font_menu = None
        self.working_dir = None
        self.show_system_tray_ray = show_system_tray_ray
        # Self attr
        # Read user setting first
        read_user_setting()
        # Set language
        language_wrapper.reset_language(user_setting_dict.get("language", "English"))
        # Jedi run on thread safe
        jedi.settings.fast_parser = False
        # Jedi only show right case_insensitive
        jedi.settings.case_insensitive_completion = False
        # Project compiler if user not choose this will use which to find
        self.python_compiler = None
        # Debug mode
        self.debug_mode: bool = debug_mode
        # Windows setup
        self.id = language_wrapper.language_word_dict.get("application_name")
        if sys.platform in ["win32", "cygwin", "msys"]:
            from ctypes import windll
            windll.shell32.SetCurrentProcessExplicitAppUserModelID(self.id)
        # set python buffered
        os.environ["PYTHONUNBUFFERED"] = "1"
        # Auto save thread
        self.auto_save_thread = None
        # Encoding
        self.encoding = "utf-8"
        read_user_color_setting()
        # Font
        self.font_database = QFontDatabase()
        # TabWidget
        self.tab_widget = QTabWidget()
        self.tab_widget.setTabsClosable(True)
        self.tab_widget.setAttribute(Qt.WidgetAttribute.WA_AlwaysShowToolTips, on=False)
        self.tab_widget.tabCloseRequested.connect(self.close_tab)
        # Timer to redirect error or message
        self.redirect_timer = QTimer(self)
        self.redirect_timer.setInterval(100)
        self.redirect_timer.start()
        self.setWindowTitle(language_wrapper.language_word_dict.get("application_name"))
        self.setToolTip(language_wrapper.language_word_dict.get("application_name"))
        set_menu_bar(self)
        # Set Icon
        self.icon_path = Path(os.getcwd() + "/je_driver_icon.ico")
        self.icon = QIcon(str(self.icon_path))
        if self.icon.isNull() is False:
            self.setWindowIcon(self.icon)
            if ExtendSystemTray.isSystemTrayAvailable() and self.show_system_tray_ray:
                self.system_tray = ExtendSystemTray(main_window=self)
                self.system_tray.setIcon(self.icon)
                self.system_tray.setVisible(True)
                self.system_tray.show()
                self.system_tray.setToolTip(language_wrapper.language_word_dict.get("application_name"))
        # Put Redirect on last to trace exception
        redirect_manager_instance.restore_std()
        redirect_manager_instance.set_redirect()
        # Timer to redirect error or message
        self.redirect_timer = QTimer(self)
        self.redirect_timer.setInterval(100)
        self.redirect_timer.timeout.connect(self.redirect)
        self.redirect_timer.start()
        # TAB Add
        self.tab_widget.addTab(EditorWidget(self), language_wrapper.language_word_dict.get("tab_name_editor"))
        self.tab_widget.addTab(
            FrontEngineMainUI(show_system_tray_ray=False, redirect_output=False),
            language_wrapper.language_word_dict.get("tab_name_frontengine"))
        self.tab_widget.addTab(BrowserWidget(), language_wrapper.language_word_dict.get("tab_name_web_browser"))
        self.tab_widget.addTab(
            BrowserWidget(start_url="https://stackoverflow.com/", search_prefix="https://stackoverflow.com/search?q="),
            language_wrapper.language_word_dict.get("tab_menu_stackoverflow_tab_name"))
        for widget_name, widget in EDITOR_EXTEND_TAB.items():
            self.tab_widget.addTab(widget(), widget_name)
        self.setCentralWidget(self.tab_widget)
        # Read Setting
        self.startup_setting()
        # If debug open 10s and close
        if self.debug_mode:
            close_timer = QTimer(self)
            close_timer.setInterval(10000)
            close_timer.timeout.connect(self.debug_close)
            close_timer.start()

    def clear_code_result(self):
        jeditor_logger.info(f"EditorMain clear_code_result")
        widget = self.tab_widget.currentWidget()
        if isinstance(widget, EditorWidget):
            widget.code_result.clear()

    def redirect(self) -> None:
        jeditor_logger.info(f"EditorMain redirect")
        for code_editor in range(self.tab_widget.count()):
            widget = self.tab_widget.widget(code_editor)
            if isinstance(widget, EditorWidget):
                # Pull out redirect text and put text in code result area
                if not redirect_manager_instance.std_out_queue.empty():
                    output_message = redirect_manager_instance.std_out_queue.get_nowait()
                    output_message = str(output_message).strip()
                    if output_message:
                        widget.code_result.append(output_message)
                widget.code_result.setTextColor(Qt.GlobalColor.red)
                if not redirect_manager_instance.std_err_queue.empty():
                    error_message = redirect_manager_instance.std_err_queue.get_nowait()
                    error_message = str(error_message).strip()
                    if error_message:
                        widget.code_result.append(error_message)
                widget.code_result.setTextColor(Qt.GlobalColor.black)
                break

    def startup_setting(self) -> None:
        jeditor_logger.info(f"EditorMain startup_setting")
        # Set font and font size, then try to open last edit file
        self.setStyleSheet(
            f"font-size: {user_setting_dict.get('ui_font_size', 12)}pt;"
            f"font-family: {user_setting_dict.get('ui_font', 'Lato')};"
        )
        # User setting
        for code_editor_count in range(self.tab_widget.count()):
            widget = self.tab_widget.widget(code_editor_count)
            if isinstance(widget, EditorWidget):
                # Font size
                widget.code_edit.setStyleSheet(
                    f"font-size: {user_setting_dict.get('font_size', 12)}pt;"
                    f"font-family: {user_setting_dict.get('font', 'Lato')};"
                )
                # Font
                widget.code_result.setStyleSheet(
                    f"font-size: {user_setting_dict.get('font_size', 12)}pt;"
                    f"font-family: {user_setting_dict.get('font', 'Lato')};"
                )
                # Default compiler
                self.python_compiler = user_setting_dict.get("python_compiler", None)
                # Last edit file
                last_file = user_setting_dict.get("last_file", None)
                if last_file is not None:
                    last_file_path = pathlib.Path(last_file)
                    if last_file_path.is_file() and last_file_path.exists() and widget.code_save_thread is None:
                        init_new_auto_save_thread(str(last_file_path), widget)
                        widget.code_edit.setPlainText(read_file(widget.current_file)[1])
                        widget.code_edit.current_file = widget.current_file
                        widget.code_edit.reset_highlighter()
                        file_is_open_manager_dict.update({str(last_file_path): str(last_file_path.name)})
                        widget.rename_self_tab()

        # Style
        self.apply_stylesheet(self, user_setting_dict.get("ui_style", "dark_amber.xml"))
        # Color
        update_actually_color_dict()

    def go_to_new_tab(self, file_path: Path):
        jeditor_logger.info(f"EditorMain go_to_new_tab file_path: {file_path}")
        if file_is_open_manager_dict.get(str(file_path), None) is None:
            editor_widget = EditorWidget(self)
            self.tab_widget.addTab(
                editor_widget,
                f"{language_wrapper.language_word_dict.get('tab_menu_editor_tab_name')} "
                f"{self.tab_widget.count()}")
            self.tab_widget.setCurrentWidget(editor_widget)
            editor_widget.open_an_file(file_path)
        else:
            widget: QWidget = self.tab_widget.findChild(EditorWidget, str(file_path))
            self.tab_widget.setCurrentWidget(widget)

    def closeEvent(self, event) -> None:
        jeditor_logger.info("EditorMain closeEvent")
        write_user_setting()
        write_user_color_setting()
        super().closeEvent(event)

    def event(self, event: QEvent) -> bool:
        jeditor_logger.info(f"EditorMain event: {event}")
        if event.type() == QEvent.Type.ToolTip:
            event.ignore()
            return False
        else:
            return super().event(event)

    def close_tab(self, index: int):
        widget = self.tab_widget.widget(index)
        if widget:
            widget.close()
        self.tab_widget.removeTab(index)

    @classmethod
    def debug_close(cls):
        sys.exit(0)



from je_editor.utils.logging.loggin_instance import jeditor_logger
from je_editor.utils.multi_language.english import english_word_dict
from je_editor.utils.multi_language.traditional_chinese import traditional_chinese_word_dict


class LanguageWrapper(object):

    def __init__(
            self
    ):
        jeditor_logger.info("Init LanguageWrapper")
        self.language: str = "English"
        self.choose_language_dict = {
            "English": english_word_dict,
            "Traditional_Chinese": traditional_chinese_word_dict
        }
        self.language_word_dict: dict = self.choose_language_dict.get(self.language)

    def reset_language(self, language) -> None:
        jeditor_logger.info(f"LanguageWrapper reset_language language: {language}")
        if language in [
            "English",
            "Traditional_Chinese"
        ]:
            self.language = language
            self.language_word_dict = self.choose_language_dict.get(self.language)


language_wrapper = LanguageWrapper()



from __future__ import annotations

from typing import TYPE_CHECKING

from je_editor.pyside_ui.main_ui.save_settings.user_setting_file import user_setting_dict
from je_editor.utils.encodings.python_encodings import python_encodings_list
from je_editor.utils.logging.loggin_instance import jeditor_logger
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain
from PySide6.QtGui import QAction, QFontDatabase

from je_editor.pyside_ui.dialog.file_dialog.create_file_dialog import CreateFileDialog
from je_editor.pyside_ui.dialog.file_dialog.open_file_dialog import choose_file_get_open_file_path, \
    choose_dir_get_dir_path
from je_editor.pyside_ui.dialog.file_dialog.save_file_dialog import choose_file_get_save_file_path


def set_file_menu(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info("build_file_menu.py add_dock_widget "
                        f"ui_we_want_to_set: {ui_we_want_to_set}")
    ui_we_want_to_set.file_menu = ui_we_want_to_set.menu.addMenu(
        language_wrapper.language_word_dict.get("file_menu_label"))
    ui_we_want_to_set.file_menu.new_file_action = QAction(
        language_wrapper.language_word_dict.get("file_menu_new_file_label"))
    ui_we_want_to_set.file_menu.new_file_action.setShortcut(
        "Ctrl+n"
    )
    ui_we_want_to_set.file_menu.new_file_action.triggered.connect(
        lambda: show_create_file_dialog(ui_we_want_to_set)
    )
    ui_we_want_to_set.file_menu.addAction(ui_we_want_to_set.file_menu.new_file_action)
    ui_we_want_to_set.file_menu.open_file_action = QAction(
        language_wrapper.language_word_dict.get("file_menu_open_file_label"))
    ui_we_want_to_set.file_menu.open_file_action.setShortcut(
        "Ctrl+o"
    )
    ui_we_want_to_set.file_menu.open_file_action.triggered.connect(
        lambda: choose_file_get_open_file_path(parent_qt_instance=ui_we_want_to_set)
    )
    ui_we_want_to_set.file_menu.addAction(ui_we_want_to_set.file_menu.open_file_action)
    ui_we_want_to_set.file_menu.open_folder_action = QAction(
        language_wrapper.language_word_dict.get("file_menu_open_folder_label"))
    ui_we_want_to_set.file_menu.open_folder_action.setShortcut(
        "Ctrl+K"
    )
    ui_we_want_to_set.file_menu.open_folder_action.triggered.connect(
        lambda: choose_dir_get_dir_path(parent_qt_instance=ui_we_want_to_set)
    )
    ui_we_want_to_set.file_menu.addAction(ui_we_want_to_set.file_menu.open_folder_action)
    ui_we_want_to_set.file_menu.save_file_action = QAction(
        language_wrapper.language_word_dict.get("file_menu_save_file_label"))
    ui_we_want_to_set.file_menu.save_file_action.setShortcut(
        "Ctrl+s"
    )
    ui_we_want_to_set.file_menu.save_file_action.triggered.connect(
        lambda: choose_file_get_save_file_path(parent_qt_instance=ui_we_want_to_set)
    )
    ui_we_want_to_set.file_menu.addAction(ui_we_want_to_set.file_menu.save_file_action)
    # Main UI font
    add_font_menu(ui_we_want_to_set)
    add_font_size_menu(ui_we_want_to_set)
    # Encoding
    add_encoding_menu(ui_we_want_to_set)


def add_encoding_menu(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info("build_file_menu.py add_encoding_menu "
                        f"ui_we_want_to_set: {ui_we_want_to_set}")
    ui_we_want_to_set.file_menu.encoding_menu = ui_we_want_to_set.file_menu.addMenu(
        language_wrapper.language_word_dict.get("file_menu_encoding_label"))
    for encoding in python_encodings_list:
        encoding_action = QAction(encoding, parent=ui_we_want_to_set.file_menu.encoding_menu)
        encoding_action.triggered.connect(
            lambda checked=False, action=encoding_action: set_encoding(ui_we_want_to_set, action))
        ui_we_want_to_set.file_menu.encoding_menu.addAction(encoding_action)


def set_encoding(ui_we_want_to_set: EditorMain, action: QAction) -> None:
    jeditor_logger.info("build_file_menu.py set_encoding "
                        f"ui_we_want_to_set: {ui_we_want_to_set} "
                        f"action: {action}")
    ui_we_want_to_set.encoding = action.text()
    user_setting_dict.update({"encoding": action.text()})


def show_create_file_dialog(ui_we_want_to_set: EditorMain):
    jeditor_logger.info("build_file_menu.py show_create_file_dialog "
                        f"ui_we_want_to_set: {ui_we_want_to_set}")
    ui_we_want_to_set.create_file_dialog = CreateFileDialog()
    ui_we_want_to_set.create_file_dialog.show()


def add_font_menu(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info("build_file_menu.py add_font_menu "
                        f"ui_we_want_to_set: {ui_we_want_to_set}")
    ui_we_want_to_set.file_menu.font_menu = ui_we_want_to_set.file_menu.addMenu(
        language_wrapper.language_word_dict.get("file_menu_font_label"))
    for family in QFontDatabase().families():
        font_action = QAction(family, parent=ui_we_want_to_set.file_menu.font_menu)
        font_action.triggered.connect(lambda checked=False, action=font_action: set_font(ui_we_want_to_set, action))
        ui_we_want_to_set.file_menu.font_menu.addAction(font_action)


def set_font(ui_we_want_to_set: EditorMain, action: QAction) -> None:
    jeditor_logger.info("build_file_menu.py set_font "
                        f"ui_we_want_to_set: {ui_we_want_to_set}")
    ui_we_want_to_set.setStyleSheet(
        f"font-size: {ui_we_want_to_set.font().pointSize()}pt;"
        f"font-family: {action.text()};"
    )
    user_setting_dict.update({"ui_font": action.text()})


def add_font_size_menu(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info("build_file_menu.py add_font_size_menu "
                        f"ui_we_want_to_set: {ui_we_want_to_set}")
    ui_we_want_to_set.file_menu.font_size_menu = ui_we_want_to_set.file_menu.addMenu(
        language_wrapper.language_word_dict.get("file_menu_font_size_label"))
    for size in range(12, 38, 2):
        font_action = QAction(str(size), parent=ui_we_want_to_set.file_menu)
        font_action.triggered.connect(
            lambda checked=False, action=font_action: set_font_size(ui_we_want_to_set, action))
        ui_we_want_to_set.file_menu.font_size_menu.addAction(font_action)


def set_font_size(ui_we_want_to_set: EditorMain, action: QAction) -> None:
    jeditor_logger.info("build_file_menu.py set_font_size "
                        f"ui_we_want_to_set: {ui_we_want_to_set} "
                        f"action: {action}")
    ui_we_want_to_set.setStyleSheet(
        f"font-size: {int(action.text())}pt;"
        f"font-family: {ui_we_want_to_set.font().family()};"
    )
    user_setting_dict.update({"ui_font_size": int(action.text())})



from __future__ import annotations

from typing import TYPE_CHECKING

from PySide6.QtWidgets import QMessageBox

from je_editor.utils.logging.loggin_instance import jeditor_logger
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain


def please_close_current_running_messagebox(ui_we_want_to_set: EditorMain):
    jeditor_logger.info(f"utils.py please_close_current_running_messagebox ui_we_want_to_set: {ui_we_want_to_set}")
    please_stop_current_running_program_messagebox = QMessageBox(ui_we_want_to_set)
    please_stop_current_running_program_messagebox.setText(
        language_wrapper.language_word_dict.get("please_stop_current_running_program")
    )
    please_stop_current_running_program_messagebox.show()



from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING

from PySide6.QtCore import Qt
from PySide6.QtGui import QAction
from PySide6.QtWidgets import QFileDialog
from frontengine import FrontEngineMainUI

from je_editor.pyside_ui.browser.browser_widget import BrowserWidget
from je_editor.pyside_ui.main_ui.dock.destroy_dock import DestroyDock
from je_editor.pyside_ui.main_ui.editor.editor_widget_dock import FullEditorWidget
from je_editor.pyside_ui.main_ui.ipython_widget.rich_jupyter import IpythonWidget
from je_editor.utils.file.open.open_file import read_file
from je_editor.utils.logging.loggin_instance import jeditor_logger
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain


def set_dock_menu(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_dock_menu.py set_dock_menu ui_we_want_to_set: {ui_we_want_to_set}")
    # Browser
    ui_we_want_to_set.dock_menu = ui_we_want_to_set.menu.addMenu(
        language_wrapper.language_word_dict.get("dock_menu_label"))
    ui_we_want_to_set.dock_menu.new_dock_browser_action = QAction(
        language_wrapper.language_word_dict.get("dock_browser_label"))
    ui_we_want_to_set.dock_menu.new_dock_browser_action.triggered.connect(
        lambda: add_dock_widget(ui_we_want_to_set)
    )
    ui_we_want_to_set.dock_menu.addAction(ui_we_want_to_set.dock_menu.new_dock_browser_action)
    # Stackoverflow
    ui_we_want_to_set.dock_menu.new_dock_stackoverflow_action = QAction(
        language_wrapper.language_word_dict.get("dock_stackoverflow_label"))
    ui_we_want_to_set.dock_menu.new_dock_stackoverflow_action.triggered.connect(
        lambda: add_dock_widget(ui_we_want_to_set, "stackoverflow")
    )
    ui_we_want_to_set.dock_menu.addAction(ui_we_want_to_set.dock_menu.new_dock_stackoverflow_action)
    # Editor
    ui_we_want_to_set.dock_menu.new_tab_dock_editor_action = QAction(
        language_wrapper.language_word_dict.get("dock_editor_label"))
    ui_we_want_to_set.dock_menu.new_tab_dock_editor_action.triggered.connect(
        lambda: add_dock_widget(ui_we_want_to_set, "editor")
    )
    ui_we_want_to_set.dock_menu.addAction(ui_we_want_to_set.dock_menu.new_tab_dock_editor_action)
    # FrontEngine
    ui_we_want_to_set.dock_menu.new_frontengine = QAction(
        language_wrapper.language_word_dict.get("dock_frontengine_label"))
    ui_we_want_to_set.dock_menu.new_frontengine.triggered.connect(
        lambda: add_dock_widget(ui_we_want_to_set, "frontengine")
    )
    ui_we_want_to_set.dock_menu.addAction(ui_we_want_to_set.dock_menu.new_frontengine)
    # Ipython
    ui_we_want_to_set.dock_menu.new_ipython = QAction(
        language_wrapper.language_word_dict.get("dock_ipython_label"))
    ui_we_want_to_set.dock_menu.new_ipython.triggered.connect(
        lambda: add_dock_widget(ui_we_want_to_set, "ipython")
    )
    ui_we_want_to_set.dock_menu.addAction(ui_we_want_to_set.dock_menu.new_ipython)


def add_dock_widget(ui_we_want_to_set: EditorMain, widget_type: str = None):
    jeditor_logger.info("build_dock_menu.py add_dock_widget "
                        f"ui_we_want_to_set: {ui_we_want_to_set} "
                        f"widget_type: {widget_type}")
    # Dock widget
    dock_widget = DestroyDock()
    if widget_type == "stackoverflow":
        dock_widget.setWindowTitle("stackoverflow")
        dock_widget.setWidget(BrowserWidget(
            start_url="https://stackoverflow.com/", search_prefix="https://stackoverflow.com/search?q="))
    elif widget_type == "editor":
        file_path = QFileDialog().getOpenFileName(
            parent=ui_we_want_to_set,
            dir=str(Path.cwd())
        )[0]
        if file_path is not None and file_path != "":
            widget = FullEditorWidget(current_file=file_path)
            file_content = read_file(file_path)[1]
            widget.code_edit.setPlainText(
                file_content
            )
            dock_widget.setWindowTitle(language_wrapper.language_word_dict.get("dock_editor_title"))
            dock_widget.setWidget(widget)
    elif widget_type == "frontengine":
        dock_widget.setWindowTitle(language_wrapper.language_word_dict.get("dock_frontengine_title"))
        dock_widget.setWidget(FrontEngineMainUI(redirect_output=False))
    elif widget_type == "ipython":
        dock_widget.setWindowTitle(language_wrapper.language_word_dict.get("dock_ipython_title"))
        dock_widget.setWidget(IpythonWidget(ui_we_want_to_set))
    else:
        dock_widget.setWindowTitle(language_wrapper.language_word_dict.get("dock_browser_title"))
        dock_widget.setWidget(BrowserWidget())
    if dock_widget.widget() is not None:
        ui_we_want_to_set.addDockWidget(Qt.DockWidgetArea.RightDockWidgetArea, dock_widget)



from os import getcwd
from pathlib import Path

from je_editor.utils.json.json_file import write_json
from je_editor.utils.logging.loggin_instance import jeditor_logger


def write_setting(save_dict: dict, file_name: str) -> None:
    jeditor_logger.info("setting_utils.py write_setting "
                        f"save_dict: {save_dict} "
                        f"file_name: {file_name}")
    save_dir = Path(getcwd() + "/.jeditor")
    save_dir.mkdir(parents=True, exist_ok=True)
    save_file = Path(getcwd() + f"/.jeditor/{file_name}")
    write_json(str(save_file), save_dict)




from __future__ import annotations

from typing import TYPE_CHECKING

from PySide6.QtGui import QAction
from PySide6.QtWidgets import QMessageBox

from je_editor.pyside_ui.main_ui.save_settings.user_setting_file import user_setting_dict
from je_editor.utils.logging.loggin_instance import jeditor_logger

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain

from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper


def set_language_menu(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_language_server.py set_language_menu ui_we_want_to_set: {ui_we_want_to_set}")
    ui_we_want_to_set.language_menu = ui_we_want_to_set.menu.addMenu(
        language_wrapper.language_word_dict.get("language_menu_label")
    )
    ui_we_want_to_set.language_menu.change_to_english_language_action = QAction(
        language_wrapper.language_word_dict.get("language_menu_bar_english")
    )
    ui_we_want_to_set.language_menu.change_to_english_language_action.triggered.connect(
        lambda: set_language("English", ui_we_want_to_set)
    )
    ui_we_want_to_set.language_menu.change_to_traditional_chinese_language_action = QAction(
        language_wrapper.language_word_dict.get("language_menu_bar_traditional_chinese")
    )
    ui_we_want_to_set.language_menu.change_to_traditional_chinese_language_action.triggered.connect(
        lambda: set_language("Traditional_Chinese", ui_we_want_to_set)
    )
    ui_we_want_to_set.language_menu.addAction(
        ui_we_want_to_set.language_menu.change_to_english_language_action
    )
    ui_we_want_to_set.language_menu.addAction(
        ui_we_want_to_set.language_menu.change_to_traditional_chinese_language_action
    )


def set_language(language: str, ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info("build_language_server.py set_language "
                        f"language: {language} "
                        f"ui_we_want_to_set: {ui_we_want_to_set}")
    language_wrapper.reset_language(language)
    user_setting_dict.update({"language": language})
    message_box = QMessageBox(ui_we_want_to_set)
    message_box.setText(language_wrapper.language_word_dict.get("language_menu_bar_please_restart_messagebox"))
    message_box.show()



from typing import List

from PySide6.QtCore import Qt
from PySide6.QtWebEngineCore import QWebEngineDownloadRequest
from PySide6.QtWebEngineWidgets import QWebEngineView

from je_editor.pyside_ui.browser.browser_download_window import BrowserDownloadWindow
from je_editor.utils.logging.loggin_instance import jeditor_logger


class BrowserView(QWebEngineView):

    def __init__(self, start_url: str = "https://www.google.com/"):
        super().__init__()
        jeditor_logger.info("Init BrowserView "
                            f"start_url: {start_url}")
        self.setUrl(start_url)
        self.download_list: List[QWebEngineDownloadRequest] = list()
        self.download_window_list: List[BrowserDownloadWindow] = list()
        self.page().profile().downloadRequested.connect(self.download_file)
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)

    def download_file(self, download_instance: QWebEngineDownloadRequest):
        jeditor_logger.info("Download File "
                            f"download_instance: {download_instance}")
        self.download_list.append(download_instance)
        download_detail_window = BrowserDownloadWindow(download_instance)
        self.download_window_list.append(download_detail_window)
        download_detail_window.show()

    def closeEvent(self, event) -> None:
        jeditor_logger.info(f"BrowserView closeEvent event: {event}")
        for download_instance in self.download_list:
            download_instance.cancel()
        for download_window in self.download_window_list:
            download_window.close()
        super().closeEvent(event)



from __future__ import annotations

from typing import TYPE_CHECKING

from je_editor.utils.logging.loggin_instance import jeditor_logger

if TYPE_CHECKING:
    from je_editor.pyside_ui.code.code_process.code_exec import ExecManager
    from je_editor.pyside_ui.code.shell_process.shell_exec import ShellManager

from typing import List, Union


class RunInstanceManager(object):

    def __init__(self):
        jeditor_logger.info("Init RunInstanceManager")
        self.instance_list: List[Union[ExecManager, ShellManager]] = list()

    def close_all_instance(self):
        jeditor_logger.info("RunInstanceManager close_all_instance")
        for manager in self.instance_list:
            if manager.process is not None:
                manager.process.terminate()
            manager.main_window.exec_program = None
            manager.main_window.exec_shell = None
            manager.main_window.exec_python_debugger = None


run_instance_manager = RunInstanceManager()



from PySide6.QtCore import Qt
from PySide6.QtWidgets import QDockWidget

from je_editor.utils.logging.loggin_instance import jeditor_logger


class DestroyDock(QDockWidget):

    def __init__(self):
        jeditor_logger.info("Init DestroyDock")
        super().__init__()
        self.setAllowedAreas(Qt.DockWidgetArea.AllDockWidgetAreas)
        # Attr
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)

    def closeEvent(self, event) -> None:
        jeditor_logger.info(f"DestroyDock closeEvent event: {event}")
        self.widget().close()
        super().close()



from __future__ import annotations

from typing import TYPE_CHECKING

from PySide6.QtCore import Qt
from PySide6.QtWidgets import QLineEdit

from je_editor.utils.logging.loggin_instance import jeditor_logger

if TYPE_CHECKING:
    from je_editor.pyside_ui.browser.browser_widget import BrowserWidget


class BrowserLineSearch(QLineEdit):

    def __init__(self, browser_widget: BrowserWidget):
        super().__init__()
        jeditor_logger.info("Init BrowserLineSearch "
                            f"browser_widget: {browser_widget}")
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        self.browser = browser_widget

    def keyPressEvent(self, event) -> None:
        if event.key() in [Qt.Key.Key_Enter, Qt.Key.Key_Return]:
            jeditor_logger.info("Browser Search")
            self.browser.search()
        super().keyPressEvent(event)



from __future__ import annotations

from typing import TYPE_CHECKING

from frontengine import FrontEngineMainUI

from je_editor.pyside_ui.browser.browser_widget import BrowserWidget
from je_editor.pyside_ui.main_ui.editor.editor_widget import EditorWidget
from je_editor.pyside_ui.main_ui.ipython_widget.rich_jupyter import IpythonWidget
from re_edge_gpt.ui.chat.main_ui import ChatMainUI

from je_editor.utils.logging.loggin_instance import jeditor_logger

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain
from PySide6.QtGui import QAction
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper


def set_tab_menu(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_tab_menu.py set_tab_menu ui_we_want_to_set:{ui_we_want_to_set}")
    # Editor
    ui_we_want_to_set.tab_menu = ui_we_want_to_set.menu.addMenu(
        language_wrapper.language_word_dict.get("tab_menu_label")
    )
    ui_we_want_to_set.tab_menu.add_editor_action = QAction(
        language_wrapper.language_word_dict.get("tab_menu_add_editor_label"))
    ui_we_want_to_set.tab_menu.add_editor_action.triggered.connect(
        lambda: add_editor_tab(ui_we_want_to_set)
    )
    # Front Engine
    ui_we_want_to_set.tab_menu.addAction(ui_we_want_to_set.tab_menu.add_editor_action)
    ui_we_want_to_set.tab_menu.add_frontengine_action = QAction(
        language_wrapper.language_word_dict.get("tab_menu_add_frontengine_label"))
    ui_we_want_to_set.tab_menu.add_frontengine_action.triggered.connect(
        lambda: add_frontengine_tab(ui_we_want_to_set)
    )
    # Web
    ui_we_want_to_set.tab_menu.addAction(ui_we_want_to_set.tab_menu.add_frontengine_action)
    ui_we_want_to_set.tab_menu.add_web_action = QAction(
        language_wrapper.language_word_dict.get("tab_menu_add_web_label"))
    ui_we_want_to_set.tab_menu.add_web_action.triggered.connect(
        lambda: add_web_tab(ui_we_want_to_set)
    )
    ui_we_want_to_set.tab_menu.addAction(ui_we_want_to_set.tab_menu.add_web_action)
    # Stackoverflow
    ui_we_want_to_set.tab_menu.add_stackoverflow_action = QAction(
        language_wrapper.language_word_dict.get("tab_menu_add_stackoverflow_label"))
    ui_we_want_to_set.tab_menu.add_stackoverflow_action.triggered.connect(
        lambda: add_stackoverflow_tab(ui_we_want_to_set)
    )
    ui_we_want_to_set.tab_menu.addAction(ui_we_want_to_set.tab_menu.add_stackoverflow_action)
    # IPython
    ui_we_want_to_set.tab_menu.add_ipython_action = QAction(
        language_wrapper.language_word_dict.get("tab_menu_ipython_tab_name"))
    ui_we_want_to_set.tab_menu.add_ipython_action.triggered.connect(
        lambda: add_ipython_tab(ui_we_want_to_set)
    )
    ui_we_want_to_set.tab_menu.addAction(ui_we_want_to_set.tab_menu.add_ipython_action)
    # ReEdgeGPT (Need fix it first)
    # ui_we_want_to_set.tab_menu.add_re_edge_gpt_action = QAction(
    #     language_wrapper.language_word_dict.get("tab_menu_re_re_edge_gpt_tab_name"))
    # ui_we_want_to_set.tab_menu.add_re_edge_gpt_action.triggered.connect(
    #     lambda: add_re_edge_gpt_tab(ui_we_want_to_set)
    # )
    # ui_we_want_to_set.tab_menu.addAction(ui_we_want_to_set.tab_menu.add_re_edge_gpt_action)


def add_editor_tab(ui_we_want_to_set: EditorMain):
    jeditor_logger.info(f"build_tab_menu.py add_editor_tab ui_we_want_to_set: {ui_we_want_to_set}")
    widget = EditorWidget(ui_we_want_to_set)
    ui_we_want_to_set.tab_widget.addTab(
        widget,
        f"{language_wrapper.language_word_dict.get('tab_menu_editor_tab_name')} "
        f"{ui_we_want_to_set.tab_widget.count()}")
    return widget


def add_frontengine_tab(ui_we_want_to_set: EditorMain):
    jeditor_logger.info(f"build_tab_menu.py add_frontengine_tab ui_we_want_to_set: {ui_we_want_to_set}")
    ui_we_want_to_set.tab_widget.addTab(
        FrontEngineMainUI(show_system_tray_ray=False, redirect_output=False),
        f"{language_wrapper.language_word_dict.get('tab_menu_frontengine_tab_name')} "
        f"{ui_we_want_to_set.tab_widget.count()}")


def add_web_tab(ui_we_want_to_set: EditorMain):
    jeditor_logger.info(f"build_tab_menu.py add_web_tab ui_we_want_to_set: {ui_we_want_to_set}")
    ui_we_want_to_set.tab_widget.addTab(
        BrowserWidget(),
        f"{language_wrapper.language_word_dict.get('tab_menu_web_tab_name')} "
        f"{ui_we_want_to_set.tab_widget.count()}")


def add_stackoverflow_tab(ui_we_want_to_set: EditorMain):
    jeditor_logger.info(f"build_tab_menu.py add_stackoverflow_tab ui_we_want_to_set: {ui_we_want_to_set}")
    ui_we_want_to_set.tab_widget.addTab(
        BrowserWidget(start_url="https://stackoverflow.com/", search_prefix="https://stackoverflow.com/search?q="),
        f"{language_wrapper.language_word_dict.get('tab_menu_stackoverflow_tab_name')} "
        f"{ui_we_want_to_set.tab_widget.count()}")


def add_ipython_tab(ui_we_want_to_set: EditorMain):
    jeditor_logger.info(f"build_tab_menu.py add_ipython_tab ui_we_want_to_set: {ui_we_want_to_set}")
    ui_we_want_to_set.tab_widget.addTab(
        IpythonWidget(ui_we_want_to_set),
        f"{language_wrapper.language_word_dict.get('tab_menu_ipython_tab_name')} "
        f"{ui_we_want_to_set.tab_widget.count()}")


def add_re_edge_gpt_tab(ui_we_want_to_set: EditorMain):
    jeditor_logger.info(f"build_tab_menu.py add_re_edge_gpt_tab ui_we_want_to_set: {ui_we_want_to_set}")
    ui_we_want_to_set.tab_widget.addTab(
        ChatMainUI(),
        f"{language_wrapper.language_word_dict.get('tab_name_re_edge_gpt')} "
        f"{ui_we_want_to_set.tab_widget.count()}")



from __future__ import annotations

from typing import TYPE_CHECKING

from je_editor.pyside_ui.main_ui.editor.editor_widget import EditorWidget
from je_editor.pyside_ui.main_ui.save_settings.user_setting_file import user_setting_dict
from je_editor.utils.logging.loggin_instance import jeditor_logger

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain
import os
from pathlib import Path

from PySide6.QtGui import QAction, QKeySequence
from PySide6.QtWidgets import QMessageBox, QInputDialog, QFileDialog

from je_editor.pyside_ui.code.shell_process.shell_exec import ShellManager

from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper


def set_venv_menu(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_venv_menu.py set_venv_menu ui_we_want_to_set: {ui_we_want_to_set}")
    ui_we_want_to_set.venv_menu = ui_we_want_to_set.menu.addMenu(
        language_wrapper.language_word_dict.get("python_env_menu_label"))
    # Create an venv
    ui_we_want_to_set.venv_menu.change_language_menu = QAction(
        language_wrapper.language_word_dict.get("python_env_menu_create_venv_label"))
    ui_we_want_to_set.venv_menu.change_language_menu.setShortcut(
        QKeySequence("Ctrl+Shift+V")
    )
    ui_we_want_to_set.venv_menu.change_language_menu.triggered.connect(
        lambda: create_venv(ui_we_want_to_set)
    )
    ui_we_want_to_set.venv_menu.addAction(ui_we_want_to_set.venv_menu.change_language_menu)
    # pip upgrade package
    ui_we_want_to_set.venv_menu.pip_upgrade_action = QAction(
        language_wrapper.language_word_dict.get("python_env_menu_pip_upgrade_label")
    )
    ui_we_want_to_set.venv_menu.pip_upgrade_action.setShortcut(
        QKeySequence("Ctrl+Shift+U")
    )
    ui_we_want_to_set.venv_menu.pip_upgrade_action.triggered.connect(
        lambda: pip_install_package_update(ui_we_want_to_set)
    )
    ui_we_want_to_set.venv_menu.addAction(ui_we_want_to_set.venv_menu.pip_upgrade_action)
    # pip package
    ui_we_want_to_set.venv_menu.pip_action = QAction(
        language_wrapper.language_word_dict.get("python_env_menu_pip_label"))
    ui_we_want_to_set.venv_menu.pip_action.setShortcut(
        QKeySequence("Ctrl+Shift+P")
    )
    ui_we_want_to_set.venv_menu.pip_action.triggered.connect(
        lambda: pip_install_package(ui_we_want_to_set)
    )
    ui_we_want_to_set.venv_menu.addAction(ui_we_want_to_set.venv_menu.pip_action)
    # choose python interpreter
    ui_we_want_to_set.venv_menu.choose_interpreter_action = QAction(
        language_wrapper.language_word_dict.get("python_env_menu_choose_interpreter_label"))
    ui_we_want_to_set.venv_menu.choose_interpreter_action.triggered.connect(
        lambda: chose_python_interpreter(ui_we_want_to_set)
    )
    ui_we_want_to_set.venv_menu.addAction(ui_we_want_to_set.venv_menu.choose_interpreter_action)


def create_venv(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_venv_menu.py create_venv ui_we_want_to_set: {ui_we_want_to_set}")
    widget = ui_we_want_to_set.tab_widget.currentWidget()
    if isinstance(widget, EditorWidget):
        widget.python_compiler = ui_we_want_to_set.python_compiler
        venv_path = Path(os.getcwd() + "/venv")
        if not venv_path.exists():
            create_venv_shell = ShellManager(main_window=widget, after_done_function=widget.code_edit.check_env,
                                             shell_encoding=ui_we_want_to_set.encoding)
            create_venv_shell.later_init()
            create_venv_shell.exec_shell(
                [f"{create_venv_shell.compiler_path}", "-m", "venv", "venv"]
            )
            widget.code_result.append(
                language_wrapper.language_word_dict.get("python_env_menu_creating_venv_message"))
        else:
            message_box = QMessageBox()
            message_box.setText(
                language_wrapper.language_word_dict.get("python_env_menu_venv_exists"))
            message_box.exec()


def shell_pip_install(ui_we_want_to_set: EditorMain, pip_install_command_list: list):
    jeditor_logger.info("build_venv_menu.py create_venv "
                        f"ui_we_want_to_set: {ui_we_want_to_set} "
                        f"pip_install_command_list: {pip_install_command_list}")
    widget = ui_we_want_to_set.tab_widget.currentWidget()
    if isinstance(widget, EditorWidget):
        widget.python_compiler = ui_we_want_to_set.python_compiler
        venv_path = Path(os.getcwd() + "/venv")
        if not venv_path.exists():
            message_box = QMessageBox()
            message_box.setText(language_wrapper.language_word_dict.get("python_env_menu_please_create_venv"))
            message_box.exec()
        else:
            ask_package_dialog = QInputDialog()
            package_text, press_ok = ask_package_dialog.getText(
                ui_we_want_to_set,
                language_wrapper.language_word_dict.get("python_env_menu_install_package_messagebox_title"),
                language_wrapper.language_word_dict.get("python_env_menu_install_package_messagebox_label")
            )
            if press_ok:
                pip_install_shell = ShellManager(main_window=widget, shell_encoding=ui_we_want_to_set.encoding)
                pip_install_shell.later_init()
                pip_install_shell.exec_shell(
                    pip_install_command_list
                )


def detect_venv() -> bool:
    jeditor_logger.info("build_venv_menu.py detect_venv")
    venv_path = Path(os.getcwd() + "/venv")
    if not venv_path.exists():
        message_box = QMessageBox()
        message_box.setText(language_wrapper.language_word_dict.get("python_env_menu_please_create_venv"))
        message_box.exec()
        return False
    return True


def pip_install_package_update(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_venv_menu.py pip_install_package_update ui_we_want_to_set: {ui_we_want_to_set}")
    widget = ui_we_want_to_set.tab_widget.currentWidget()
    if isinstance(widget, EditorWidget):
        widget.python_compiler = ui_we_want_to_set.python_compiler
        if detect_venv:
            ask_package_dialog = QInputDialog()
            package_text, press_ok = ask_package_dialog.getText(
                ui_we_want_to_set,
                language_wrapper.language_word_dict.get("python_env_menu_install_package_messagebox_title"),
                language_wrapper.language_word_dict.get("python_env_menu_install_or_update_package_messagebox_label")
            )
            if press_ok:
                pip_install_shell = ShellManager(main_window=widget, after_done_function=widget.code_edit.check_env,
                                                 shell_encoding=ui_we_want_to_set.encoding)
                pip_install_shell.later_init()
                pip_install_shell.exec_shell(
                    [f"{pip_install_shell.compiler_path}", "-m", "pip", "install", f"{package_text}", "-U"]
                )


def pip_install_package(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_venv_menu.py pip_install_package ui_we_want_to_set: {ui_we_want_to_set}")
    widget = ui_we_want_to_set.tab_widget.currentWidget()
    if isinstance(widget, EditorWidget):
        widget.python_compiler = ui_we_want_to_set.python_compiler
        if detect_venv:
            ask_package_dialog = QInputDialog()
            package_text, press_ok = ask_package_dialog.getText(
                ui_we_want_to_set,
                language_wrapper.language_word_dict.get("python_env_menu_install_package_messagebox_title"),
                language_wrapper.language_word_dict.get("python_env_menu_install_package_messagebox_label")
            )
            if press_ok:
                pip_install_shell = ShellManager(main_window=widget, after_done_function=widget.code_edit.check_env,
                                                 shell_encoding=ui_we_want_to_set.encoding)
                pip_install_shell.later_init()
                pip_install_shell.exec_shell(
                    [f"{pip_install_shell.compiler_path}", "-m", "pip", "install", f"{package_text}"]
                )


def chose_python_interpreter(ui_we_want_to_set: EditorMain):
    jeditor_logger.info(f"build_venv_menu.py chose_python_interpreter ui_we_want_to_set: {ui_we_want_to_set}")
    file_path = QFileDialog().getOpenFileName(
        parent=ui_we_want_to_set,
        dir=str(Path.cwd())
    )[0]
    if file_path is not None and file_path != "":
        ui_we_want_to_set.python_compiler = file_path
        user_setting_dict.update({"python_compiler": file_path})



import logging
import queue
import sys

from je_editor.utils.logging.loggin_instance import jeditor_logger


class RedirectStdOut(logging.Handler):

    # redirect logging std output to queue

    def __init__(self):
        super().__init__()

    def write(self, content_to_write) -> None:
        redirect_manager_instance.std_out_queue.put(content_to_write)

    def emit(self, record: logging.LogRecord) -> None:
        redirect_manager_instance.std_out_queue.put(self.format(record))


class RedirectStdErr(logging.Handler):

    # redirect logging stderr output to queue

    def __init__(self):
        super().__init__()

    def write(self, content_to_write) -> None:
        redirect_manager_instance.std_err_queue.put(content_to_write)

    def emit(self, record: logging.LogRecord) -> None:
        redirect_manager_instance.std_err_queue.put(self.format(record))


class RedirectManager(object):
    # Redirect all output to queue
    def __init__(self):
        jeditor_logger.info("Init RedirectManager")
        self.std_err_queue = queue.Queue()
        self.std_out_queue = queue.Queue()

    @staticmethod
    def set_redirect() -> None:
        """
        :return: None
        """
        jeditor_logger.info("RedirectManager set_redirect")
        redirect_out = RedirectStdOut()
        redirect_err = RedirectStdErr()
        sys.stdout = redirect_out
        sys.stderr = redirect_err
        default_logger = logging.getLogger("RedirectManager")
        default_logger.addHandler(redirect_err)
        skip_logger_list = ["JEditor", "FrontEngine", "AutomationIDE"]
        for name in logging.root.manager.loggerDict.keys():
            if name in skip_logger_list:
                continue
            else:
                logging.getLogger(name).addHandler(redirect_err)

    @staticmethod
    def restore_std() -> None:
        """
        reset redirect
        :return: None
        """
        jeditor_logger.info("RedirectManager restore_std")
        sys.stdout = sys.__stdout__
        sys.stderr = sys.__stderr__


redirect_manager_instance = RedirectManager()



from __future__ import annotations

from typing import TYPE_CHECKING

from je_editor.pyside_ui.main_ui.menu.dock_menu.build_dock_menu import set_dock_menu
from je_editor.pyside_ui.main_ui.menu.language_menu.build_language_server import set_language_menu
from je_editor.pyside_ui.main_ui.menu.style_menu.build_style_menu import set_style_menu
from je_editor.pyside_ui.main_ui.menu.tab_menu.build_tab_menu import set_tab_menu
from je_editor.pyside_ui.main_ui.menu.text_menu.build_text_menu import set_text_menu
from je_editor.utils.logging.loggin_instance import jeditor_logger

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain
from PySide6.QtWidgets import QMenuBar

from je_editor.pyside_ui.main_ui.menu.check_style_menu.build_check_style_menu import set_check_menu
from je_editor.pyside_ui.main_ui.menu.file_menu.build_file_menu import set_file_menu
from je_editor.pyside_ui.main_ui.menu.help_menu.build_help_menu import set_help_menu
from je_editor.pyside_ui.main_ui.menu.run_menu.build_run_menu import set_run_menu
from je_editor.pyside_ui.main_ui.menu.python_env_menu.build_venv_menu import set_venv_menu


def set_menu_bar(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"set_menu_bar.py set_menu_bar ui_we_want_to_set: {ui_we_want_to_set}")
    # set menu
    ui_we_want_to_set.menu = QMenuBar()
    set_file_menu(ui_we_want_to_set)
    set_run_menu(ui_we_want_to_set)
    set_text_menu(ui_we_want_to_set)
    set_check_menu(ui_we_want_to_set)
    set_help_menu(ui_we_want_to_set)
    set_venv_menu(ui_we_want_to_set)
    set_tab_menu(ui_we_want_to_set)
    set_dock_menu(ui_we_want_to_set)
    set_style_menu(ui_we_want_to_set)
    set_language_menu(ui_we_want_to_set)
    ui_we_want_to_set.setMenuBar(ui_we_want_to_set.menu)



from __future__ import annotations

from typing import TYPE_CHECKING

from je_editor.utils.logging.loggin_instance import jeditor_logger

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorWidget

from je_editor.pyside_ui.code.auto_save.auto_save_thread import CodeEditSaveThread

auto_save_manager_dict: dict = dict()

file_is_open_manager_dict: dict = dict()


def init_new_auto_save_thread(file_path: str, widget: EditorWidget):
    jeditor_logger.info(f"auto_save_manager.py init_new_auto_save_thread "
                        f"file_path: {file_path} "
                        f"widget: {widget}")
    widget.current_file = file_path
    if auto_save_manager_dict.get(file_path, None) is None:
        widget.code_save_thread = CodeEditSaveThread(
            file_to_save=widget.current_file, editor=widget.code_edit)
        auto_save_manager_dict.update({
            file_path: widget.code_save_thread
        })
        widget.code_save_thread.start()



import logging
from logging.handlers import RotatingFileHandler

logging.root.setLevel(logging.DEBUG)
jeditor_logger = logging.getLogger("JEditor")
formatter = logging.Formatter('%(asctime)s | %(name)s | %(levelname)s | %(message)s')


class JEditorLoggingHandler(RotatingFileHandler):

    # redirect logging stderr output to queue

    def __init__(self, filename: str = "JEditor.log", mode="w",
                 maxBytes:int=1073741824, backupCount:int=0):
        super().__init__(filename=filename, mode=mode, maxBytes=maxBytes, backupCount=backupCount)
        self.formatter = formatter
        self.setLevel(logging.DEBUG)

    def emit(self, record: logging.LogRecord) -> None:
        super().emit(record)


# File handler
file_handler = JEditorLoggingHandler()
jeditor_logger.addHandler(file_handler)





import sys

from PySide6.QtCore import QCoreApplication
from PySide6.QtWidgets import QApplication
from qt_material import apply_stylesheet

from je_editor.pyside_ui.main_ui.main_editor import EditorMain


def start_editor(debug_mode: bool = False) -> None:
    new_editor = QCoreApplication.instance()
    if new_editor is None:
        new_editor = QApplication(sys.argv)
    window = EditorMain(debug_mode)
    apply_stylesheet(new_editor, theme='dark_amber.xml')
    window.showMaximized()
    sys.exit(new_editor.exec())



from pathlib import Path

from PySide6.QtCore import Qt
from PySide6.QtWidgets import QWidget, QGridLayout, QScrollArea
from frontengine.utils.multi_language.language_wrapper import language_wrapper

from je_editor.pyside_ui.code.plaintext_code_edit.code_edit_plaintext import CodeEditor
from je_editor.pyside_ui.main_ui.save_settings.user_setting_file import user_setting_dict
from je_editor.utils.file.save.save_file import write_file
from je_editor.utils.logging.loggin_instance import jeditor_logger


class FullEditorWidget(QWidget):

    def __init__(self, current_file: str):
        jeditor_logger.info(f"Init FullEditorWidget current_file: {current_file}")
        super().__init__()
        # Init variable
        self.current_file = current_file
        # Attr
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        # UI
        self.grid_layout = QGridLayout(self)
        self.setWindowTitle(language_wrapper.language_word_dict.get("application_name"))
        # code edit and code result plaintext
        self.code_edit = CodeEditor(self)
        self.code_edit_scroll_area = QScrollArea()
        self.code_edit_scroll_area.setWidgetResizable(True)
        self.code_edit_scroll_area.setViewportMargins(0, 0, 0, 0)
        self.code_edit_scroll_area.setWidget(self.code_edit)
        self.grid_layout.addWidget(self.code_edit_scroll_area, 0, 0)
        # Font
        self.code_edit.setStyleSheet(
            f"font-size: {user_setting_dict.get('font_size', 12)}pt;"
            f"font-family: {user_setting_dict.get('font', 'Lato')};"
        )

    def closeEvent(self, event) -> None:
        jeditor_logger.info(f"FullEditorWidget closeEvent event: {event}")
        path = Path(self.current_file)
        if path.exists() and path.is_file():
            write_file(self.current_file, self.code_edit.toPlainText())
        super().closeEvent(event)



import tokenize

import pycodestyle

from je_editor.utils.logging.loggin_instance import jeditor_logger


class PEP8FormatChecker(pycodestyle.Checker):

    def __init__(self, filename: str, **kwargs):
        jeditor_logger.info(f"Init PEP8FormatChecker "
                            f"filename: {filename} "
                            f"kwargs: {kwargs}")
        super().__init__(filename, **kwargs)
        # Init variable
        self.physical_line = None
        self.blank_before = None
        self.blank_lines = None
        self.tokens = None
        self.previous_unindented_logical_line = None
        self.previous_logical = None
        self.indent_level = None
        self.previous_indent_level = None
        self.line_number = None
        self.indent_char = None
        self.total_lines = None
        self.new_line = frozenset([tokenize.NL, tokenize.NEWLINE])
        self.report_error = self.replace_report_error
        self.current_file: str = filename
        self.error_list: list = list()

    def replace_report_error(self, line_number, offset, text, check):
        jeditor_logger.info(f"PEP8FormatChecker replace_report_error "
                            f"line_number: {line_number} "
                            f"offset: {offset} "
                            f"text: {text}")
        if not text.startswith("W191"):
            self.error_list.append(f"{text} on line: {line_number}, offset: {offset}")

    def check_all_format(self, expected=None, line_offset=0) -> int:
        """Run all checks on the input file."""
        jeditor_logger.info(f"PEP8FormatChecker check_all_format "
                            f"expected: {expected} "
                            f"line_offset: {line_offset}")
        self.report.init_file(self.filename, self.lines, expected, line_offset)
        self.total_lines = len(self.lines)
        if self._ast_checks:
            self.check_ast()
        self.line_number = 0
        self.indent_char = None
        self.indent_level = self.previous_indent_level = 0
        self.previous_logical = ''
        self.previous_unindented_logical_line = ''
        self.tokens = []
        self.blank_lines = self.blank_before = 0
        parens = 0
        for token in self.generate_tokens():
            self.tokens.append(token)
            token_type, text = token[0:2]
            if self.verbose >= 3:
                if token[2][0] == token[3][0]:
                    pos = '[{}:{}]'.format(token[2][1] or '', token[3][1])
                else:
                    pos = 'l.%s' % token[3][0]
                self.replace_report_error(token[2][0], pos, tokenize.tok_name[token[0]], text)
            if token_type == tokenize.OP:
                if text in '([{':
                    parens += 1
                elif text in '}])':
                    parens -= 1
            elif not parens:
                if token_type in self.new_line:
                    if token_type == tokenize.NEWLINE:
                        self.check_logical()
                        self.blank_before = 0
                    elif len(self.tokens) == 1:
                        # The physical line contains only this token.
                        self.blank_lines += 1
                        del self.tokens[0]
                    else:
                        self.check_logical()
        if self.tokens:
            self.check_physical(self.lines[-1])
            self.check_logical()
        return self.report.get_file_results()



from __future__ import annotations

import queue
import subprocess
import sys
from pathlib import Path
from threading import Thread
from typing import Union

from PySide6.QtCore import QTimer
from PySide6.QtWidgets import QTextEdit

from je_editor.pyside_ui.code.running_process_manager import run_instance_manager
from je_editor.pyside_ui.main_ui.editor.editor_widget import EditorWidget
from je_editor.pyside_ui.main_ui.save_settings.user_color_setting_file import actually_color_dict
from je_editor.utils.exception.exception_tags import je_editor_init_error
from je_editor.utils.exception.exceptions import JEditorException
from je_editor.utils.logging.loggin_instance import jeditor_logger
from je_editor.utils.venv_check.check_venv import check_and_choose_venv


class ExecManager(object):

    def __init__(
            self,
            main_window: Union[EditorWidget, None] = None,
            program_language: str = "python",
            program_encoding: str = "utf-8",
            program_buffer: int = 1024,
    ):
        """
        :param main_window: Pyside main window
        :param program_language: which program language
        :param program_encoding: which encoding
        """
        jeditor_logger.info(f"Init ExecManager "
                            f"main_window: {main_window} "
                            f"program_language: {program_language} "
                            f"program_encoding: {program_encoding} "
                            f"program_buffer: {program_buffer}")
        self.read_program_error_output_from_thread = None
        self.read_program_output_from_thread = None
        self.main_window: EditorWidget = main_window
        self.compiler_path = None
        self.code_result: Union[QTextEdit, None] = None
        self.timer: Union[QTimer, None] = None
        self.still_run_program = True
        self.process: Union[subprocess.Popen, None] = None
        self.run_output_queue = queue.Queue()
        self.run_error_queue = queue.Queue()
        self.program_language = program_language
        self.program_encoding = program_encoding
        self.program_buffer = program_buffer
        self.renew_path()
        run_instance_manager.instance_list.append(self)

    def renew_path(self) -> None:
        jeditor_logger.info("ExecManager renew_path")
        if self.main_window.python_compiler is None:
            # Renew compiler path
            if sys.platform in ["win32", "cygwin", "msys"]:
                venv_path = Path(str(Path.cwd()) + "/venv/Scripts")
            else:
                venv_path = Path(str(Path.cwd()) + "/venv/bin")
            self.compiler_path = check_and_choose_venv(venv_path)
        else:
            self.compiler_path = self.main_window.python_compiler

    def later_init(self) -> None:
        jeditor_logger.info("ExecManager later_init")
        # Enable timer and code result area
        if self.main_window is not None:
            self.code_result: QTextEdit = self.main_window.code_result
            self.timer = QTimer(self.main_window)
        else:
            raise JEditorException(je_editor_init_error)

    def exec_code(self, exec_file_name, exec_prefix: Union[str, list] = None) -> None:
        """
        :param exec_file_name: string file will open to run
        :param exec_prefix: user define exec prefix
        :return: if error return result and True else return result and False
        """
        jeditor_logger.info(f"ExecManager exec_code "
                            f"exec_file_name: {exec_file_name} "
                            f"exec_prefix: {exec_prefix}")
        try:
            self.exit_program()
            self.code_result.setTextColor(actually_color_dict.get("normal_output_color"))
            self.code_result.setPlainText("")
            file_path = Path(exec_file_name)
            reformat_os_file_path = str(file_path.absolute())
            # detect file is exist
            exec_file = reformat_os_file_path
            # run program
            if exec_prefix is None:
                execute_program_param = [self.compiler_path, exec_file]
            else:
                if isinstance(exec_prefix, str):
                    execute_program_param = [self.compiler_path, exec_prefix, exec_file]
                else:
                    execute_program_param = list()
                    execute_program_param.append(self.compiler_path)
                    for prefix in exec_prefix:
                        execute_program_param.append(prefix)
                    execute_program_param.append(exec_file)
            if sys.platform not in ["win32", "cygwin", "msys"]:
                execute_program_param = " ".join(execute_program_param)
            self.process = subprocess.Popen(
                execute_program_param,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                stdin=subprocess.PIPE,
                shell=True
            )
            self.still_run_program = True
            # program output message queue thread
            self.read_program_output_from_thread = Thread(
                target=self.read_program_output_from_process,
                daemon=True
            )
            self.read_program_output_from_thread.start()
            # program error message queue thread
            self.read_program_error_output_from_thread = Thread(
                target=self.read_program_error_output_from_process,
                daemon=True
            )
            self.read_program_error_output_from_thread.start()
            # show which file execute
            self.code_result.append(self.compiler_path + " " + reformat_os_file_path)
            # start tkinter_ui update
            # start timer
            self.timer = QTimer(self.main_window)
            self.timer.setInterval(100)
            self.timer.timeout.connect(self.pull_text)
            self.timer.start()
        except Exception as error:
            self.code_result.setTextColor(actually_color_dict.get("error_output_color"))
            self.code_result.append(str(error))
            self.code_result.setTextColor(actually_color_dict.get("normal_output_color"))
            if self.process is not None:
                self.process.terminate()

    def full_exit_program(self):
        jeditor_logger.info("ExecManager full_exit_program")
        self.timer.stop()
        self.exit_program()
        self.main_window.exec_program = None

    def pull_text(self) -> None:
        jeditor_logger.info("ExecManager pull_text")
        # Pull text from queue and put in code result area
        try:
            self.code_result.setTextColor(actually_color_dict.get("normal_output_color"))
            if not self.run_output_queue.empty():
                output_message = self.run_output_queue.get_nowait()
                output_message = str(output_message).strip()
                if output_message:
                    self.code_result.append(output_message)
            self.code_result.setTextColor(actually_color_dict.get("error_output_color"))
            if not self.run_error_queue.empty():
                error_message = self.run_error_queue.get_nowait()
                error_message = str(error_message).strip()
                if error_message:
                    self.code_result.append(error_message)
            self.code_result.setTextColor(actually_color_dict.get("normal_output_color"))
        except queue.Empty:
            pass
        if self.process.returncode == 0:
            self.full_exit_program()
        elif self.process.returncode is not None:
            self.full_exit_program()
        if self.still_run_program:
            # poll return code
            self.process.poll()

    # Exit program change run flag to false and clean read thread and queue and process
    def exit_program(self) -> None:
        jeditor_logger.info("ExecManager exit_program")
        self.still_run_program = False
        if self.read_program_output_from_thread is not None:
            self.read_program_output_from_thread = None
        if self.read_program_error_output_from_thread is not None:
            self.read_program_error_output_from_thread = None
        self.print_and_clear_queue()
        if self.process is not None:
            self.process.terminate()
            self.code_result.append(f"Program exit with code {self.process.returncode}")
            self.process = None

    # Pull all remain string on queue and add to code result area
    def print_and_clear_queue(self) -> None:
        jeditor_logger.info("ExecManager print_and_clear_queue")
        self.run_output_queue = queue.Queue()
        self.run_error_queue = queue.Queue()

    def read_program_output_from_process(self) -> None:
        jeditor_logger.info("ExecManager read_program_output_from_process")
        while self.still_run_program:
            program_output_data: str = self.process.stdout.read(
                self.program_buffer).decode(self.program_encoding, "replace")
            if self.process:
                self.process.stdout.flush()
            self.run_output_queue.put_nowait(program_output_data)

    def read_program_error_output_from_process(self) -> None:
        jeditor_logger.info("ExecManager read_program_error_output_from_process")
        while self.still_run_program:
            program_error_output_data: str = self.process.stderr.read(
                self.program_buffer).decode(self.program_encoding, "replace")
            if self.process:
                self.process.stderr.flush()
            self.run_error_queue.put_nowait(program_error_output_data)



from __future__ import annotations

from typing import TYPE_CHECKING

from IPython.lib import guisupport
from PySide6.QtWidgets import QWidget, QGridLayout
from qtconsole.inprocess import QtInProcessKernelManager
from qtconsole.rich_jupyter_widget import RichJupyterWidget

from je_editor.utils.logging.loggin_instance import jeditor_logger

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain


class IpythonWidget(QWidget):

    def __init__(self, main_window: EditorMain):
        jeditor_logger.info(f"Init IpythonWidget main_window: {main_window}")
        super().__init__()
        self.grid_layout = QGridLayout()
        app = guisupport.get_app_qt4()
        self.kernel_manager = QtInProcessKernelManager()
        self.kernel_manager.start_kernel()
        self.kernel = self.kernel_manager.kernel
        self.kernel_client = self.kernel_manager.client()
        self.kernel_client.start_channels()
        self.jupyter_widget = RichJupyterWidget()
        self.jupyter_widget.kernel_manager = self.kernel_manager
        self.jupyter_widget.kernel_client = self.kernel_client
        self.grid_layout.addWidget(self.jupyter_widget, 0, 0, -1, -1)
        guisupport.start_event_loop_qt4(app)

        self.setLayout(self.grid_layout)

    def close(self):
        jeditor_logger.info("IpythonWidget close")
        if self.kernel_client:
            self.kernel_client.stop_channels()
        if self.kernel_manager:
            self.kernel_manager.restart_kernel()
            self.kernel_manager.shutdown_kernel()
        super().close()



from __future__ import annotations

from typing import TYPE_CHECKING

from je_editor.pyside_ui.main_ui.editor.editor_widget import EditorWidget
from je_editor.utils.logging.loggin_instance import jeditor_logger
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain
from PySide6.QtGui import QAction, QKeySequence
from yapf.yapflib.yapf_api import FormatCode

from je_editor.utils.json_format.json_process import reformat_json


def set_check_menu(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_check_style_menu.py set_check_menu ui_we_want_to_set: {ui_we_want_to_set}")
    ui_we_want_to_set.check_menu = ui_we_want_to_set.menu.addMenu(
        language_wrapper.language_word_dict.get("check_code_style_menu_label"))
    # Yapf code check
    ui_we_want_to_set.check_menu.yapf_check_python_action = QAction(
        language_wrapper.language_word_dict.get("yapf_reformat_label"))
    ui_we_want_to_set.check_menu.yapf_check_python_action.setShortcut(
        QKeySequence("Ctrl+Shift+Y"))
    ui_we_want_to_set.check_menu.yapf_check_python_action.triggered.connect(
        lambda: yapf_check_python_code(
            ui_we_want_to_set
        )
    )
    ui_we_want_to_set.check_menu.addAction(ui_we_want_to_set.check_menu.yapf_check_python_action)
    # Reformat JSON
    ui_we_want_to_set.check_menu.reformat_json_action = QAction(
        language_wrapper.language_word_dict.get("reformat_json_label"))
    ui_we_want_to_set.check_menu.reformat_json_action.setShortcut("Ctrl+j")
    ui_we_want_to_set.check_menu.reformat_json_action.triggered.connect(
        lambda: reformat_json_text(
            ui_we_want_to_set
        )
    )
    ui_we_want_to_set.check_menu.addAction(ui_we_want_to_set.check_menu.reformat_json_action)


def yapf_check_python_code(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_check_style_menu.py yapf_check_python_code ui_we_want_to_set: {ui_we_want_to_set}")
    widget = ui_we_want_to_set.tab_widget.currentWidget()
    if isinstance(widget, EditorWidget):
        code_text = widget.code_edit.toPlainText()
        widget.code_result.setPlainText("")
        format_code = FormatCode(
            unformatted_source=code_text,
            style_config="google"
        )
        if isinstance(format_code, tuple):
            widget.code_edit.setPlainText(format_code[0])


def reformat_json_text(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_check_style_menu.py reformat_json_text ui_we_want_to_set: {ui_we_want_to_set}")
    widget = ui_we_want_to_set.tab_widget.currentWidget()
    if isinstance(widget, EditorWidget):
        code_text = widget.code_edit.toPlainText()
        widget.code_result.setPlainText("")
        widget.code_edit.setPlainText(reformat_json(code_text))



from __future__ import annotations

from typing import TYPE_CHECKING

from je_editor.pyside_ui.browser.browser_widget import BrowserWidget
from je_editor.utils.logging.loggin_instance import jeditor_logger

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain

from PySide6.QtGui import QAction
from PySide6.QtWidgets import QMessageBox

from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper


def set_help_menu(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info("build_help_menu.py set_help_menu "
                        f"ui_we_want_to_set: {ui_we_want_to_set}")
    ui_we_want_to_set.help_menu = ui_we_want_to_set.menu.addMenu(
        language_wrapper.language_word_dict.get("help_menu_label"))
    ui_we_want_to_set.help_menu.help_github_action = QAction(
        language_wrapper.language_word_dict.get("help_menu_open_github_label"))
    ui_we_want_to_set.help_menu.help_github_action.triggered.connect(
        lambda: open_web_browser(
            ui_we_want_to_set,
            "https://github.com/Integrated-Testing-Environment/je_editor",
            language_wrapper.language_word_dict.get("help_menu_open_github_label"))
    )
    ui_we_want_to_set.help_menu.addAction(ui_we_want_to_set.help_menu.help_github_action)

    ui_we_want_to_set.help_menu.help_doc_action = QAction(
        language_wrapper.language_word_dict.get("help_menu_open_doc_label"))
    ui_we_want_to_set.help_menu.help_doc_action.triggered.connect(
        lambda: open_web_browser(
            ui_we_want_to_set,
            "https://je-editor.readthedocs.io/en/latest/",
            language_wrapper.language_word_dict.get("help_menu_open_doc_label"))
    )
    ui_we_want_to_set.help_menu.addAction(ui_we_want_to_set.help_menu.help_doc_action)

    ui_we_want_to_set.help_menu.help_about_action = QAction(
        language_wrapper.language_word_dict.get("help_menu_open_about_label"))
    ui_we_want_to_set.help_menu.help_about_action.triggered.connect(
        show_about
    )
    ui_we_want_to_set.help_menu.addAction(ui_we_want_to_set.help_menu.help_about_action)
    # Open Doc
    ui_we_want_to_set.help_menu.open_bing_gpt_menu_doc_action = QAction(
        language_wrapper.language_word_dict.get("help_menu_open_re_edge_gpt_doc_label"))
    ui_we_want_to_set.help_menu.open_bing_gpt_menu_doc_action.triggered.connect(
        lambda: open_web_browser(
            ui_we_want_to_set,
            "https://reedgegpt.readthedocs.io/en/latest/",
            "ReEdgeGPT Doc"
        )
    )
    ui_we_want_to_set.help_menu.addAction(
        ui_we_want_to_set.help_menu.open_bing_gpt_menu_doc_action
    )
    # Open Github
    ui_we_want_to_set.help_menu.open_re_edge_gpt_github_action = QAction(
        language_wrapper.language_word_dict.get("help_menu_open_re_edge_gpt_github_label"))
    ui_we_want_to_set.help_menu.open_re_edge_gpt_github_action.triggered.connect(
        lambda: open_web_browser(
            ui_we_want_to_set,
            "https://github.com/Integration-Automation/ReEdgeGPT",
            "ReEdgeGPT GitHub"
        )
    )
    ui_we_want_to_set.help_menu.addAction(
        ui_we_want_to_set.help_menu.open_re_edge_gpt_github_action
    )


def open_web_browser(ui_we_want_to_set: EditorMain, url: str, tab_name: str):
    jeditor_logger.info("build_help_menu.py open_web_browser "
                        f"ui_we_want_to_set: {ui_we_want_to_set} "
                        f"url: {url} "
                        f"tab_name: {tab_name}")
    ui_we_want_to_set.tab_widget.addTab(
        BrowserWidget(start_url=url),
        f"{tab_name}{ui_we_want_to_set.tab_widget.count()}"
    )


def show_about():
    jeditor_logger.info("build_help_menu.py show_about")
    message_box = QMessageBox()
    message_box.setText(
        """
JEditor
Create by JE-Chen 2020 ~ Now
        """
    )
    message_box.exec()



from __future__ import annotations

from typing import TYPE_CHECKING

from PySide6.QtGui import QAction

from je_editor.pyside_ui.main_ui.editor.editor_widget import EditorWidget
from je_editor.pyside_ui.main_ui.save_settings.user_setting_file import user_setting_dict
from je_editor.utils.logging.loggin_instance import jeditor_logger

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper


def set_text_menu(ui_we_want_to_set: EditorMain):
    jeditor_logger.info(f"build_text_menu.py set_text_menu ui_we_want_to_set: {ui_we_want_to_set}")
    ui_we_want_to_set.text_menu = ui_we_want_to_set.menu.addMenu(
        language_wrapper.language_word_dict.get("text_menu_label"))
    ui_we_want_to_set.text_menu.font_menu = ui_we_want_to_set.text_menu.addMenu(
        language_wrapper.language_word_dict.get("text_menu_label_font"))
    for family in ui_we_want_to_set.font_database.families():
        font_action = QAction(family, parent=ui_we_want_to_set.text_menu.font_menu)
        font_action.triggered.connect(
            lambda checked=False, action=font_action: set_font(ui_we_want_to_set, action))
        ui_we_want_to_set.text_menu.font_menu.addAction(font_action)
    ui_we_want_to_set.text_menu.font_size_menu = ui_we_want_to_set.text_menu.addMenu(
        language_wrapper.language_word_dict.get("text_menu_label_font_size"))
    for size in range(12, 38, 2):
        font_action = QAction(str(size), parent=ui_we_want_to_set.text_menu.font_size_menu)
        font_action.triggered.connect(
            lambda checked=False, action=font_action: set_font_size(ui_we_want_to_set, action))
        ui_we_want_to_set.text_menu.font_size_menu.addAction(font_action)


def set_font(ui_we_want_to_set: EditorMain, action: QAction) -> None:
    jeditor_logger.info("build_text_menu.py set_font "
                        f"ui_we_want_to_set: {ui_we_want_to_set} "
                        f"action: {action}")
    for code_editor in range(ui_we_want_to_set.tab_widget.count()):
        widget = ui_we_want_to_set.tab_widget.widget(code_editor)
        if isinstance(widget, EditorWidget):
            widget.code_edit.setStyleSheet(
                f"font-size: {widget.code_edit.font().pointSize()}pt;"
                f"font-family: {action.text()};"
            )
            widget.code_result.setStyleSheet(
                f"font-size: {widget.code_result.font().pointSize()}pt;"
                f"font-family: {action.text()};"
            )
            user_setting_dict.update({"font": action.text()})


def set_font_size(ui_we_want_to_set: EditorMain, action: QAction) -> None:
    jeditor_logger.info("build_text_menu.py set_font_size "
                        f"ui_we_want_to_set: {ui_we_want_to_set} "
                        f"action: {action}")
    for code_editor in range(ui_we_want_to_set.tab_widget.count()):
        widget = ui_we_want_to_set.tab_widget.widget(code_editor)
        if type(widget) is EditorWidget:
            widget.code_edit.setStyleSheet(
                f"font-size: {int(action.text())}pt;"
                f"font-family: {widget.code_edit.font().family()};"
            )
            widget.code_result.setStyleSheet(
                f"font-size: {int(action.text())}pt;"
                f"font-family: {widget.code_result.font().family()};"
            )
            user_setting_dict.update({"font_size": int(action.text())})



from __future__ import annotations

from typing import TYPE_CHECKING

from je_editor.utils.logging.loggin_instance import jeditor_logger

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain
import sys

from PySide6.QtGui import QAction
from PySide6.QtWidgets import QSystemTrayIcon, QMenu
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper


class ExtendSystemTray(QSystemTrayIcon):

    def __init__(self, main_window: EditorMain):
        jeditor_logger.info(f"Init ExtendSystemTray main_window: {main_window}")
        super().__init__(parent=main_window)
        self.menu = QMenu()
        self.main_window = main_window
        self.hide_main_window_action = QAction(
            language_wrapper.language_word_dict.get("system_tray_hide"))
        self.hide_main_window_action.triggered.connect(self.main_window.hide)
        self.menu.addAction(self.hide_main_window_action)
        self.maximized_main_window_action = QAction(
            language_wrapper.language_word_dict.get("system_tray_maximized"))
        self.maximized_main_window_action.triggered.connect(self.main_window.showMaximized)
        self.menu.addAction(self.maximized_main_window_action)
        self.normal_main_window_action = QAction(
            language_wrapper.language_word_dict.get("system_tray_normal"))
        self.normal_main_window_action.triggered.connect(self.main_window.showNormal)
        self.menu.addAction(self.normal_main_window_action)
        self.close_main_window_action = QAction(
            language_wrapper.language_word_dict.get("system_tray_close"))
        self.close_main_window_action.triggered.connect(self.close_all)
        self.menu.addAction(self.close_main_window_action)
        self.setContextMenu(self.menu)
        self.activated.connect(self.clicked)

    def close_all(self):
        jeditor_logger.info("ExtendSystemTray close_all")
        self.setVisible(False)
        self.main_window.close()
        sys.exit(0)

    def clicked(self, reason):
        if reason == self.ActivationReason.DoubleClick:
            jeditor_logger.info("ExtendSystemTray DoubleClick")
            self.main_window.showMaximized()



from __future__ import annotations

from typing import TYPE_CHECKING

from je_editor.pyside_ui.main_ui.editor.editor_widget import EditorWidget
from je_editor.pyside_ui.main_ui.editor.process_input import ProcessInput
from je_editor.pyside_ui.main_ui.menu.run_menu.under_run_menu.utils import please_close_current_running_messagebox
from je_editor.utils.logging.loggin_instance import jeditor_logger

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain
from PySide6.QtGui import QAction

from je_editor.pyside_ui.code.code_process.code_exec import ExecManager
from je_editor.pyside_ui.dialog.file_dialog.save_file_dialog import choose_file_get_save_file_path
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper


def set_debug_menu(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_debug_menu.py set_debug_menu ui_we_want_to_set: {ui_we_want_to_set}")
    # Debug menu inside run menu
    ui_we_want_to_set.debug_menu = ui_we_want_to_set.run_menu.addMenu(
        language_wrapper.language_word_dict.get("editor_debugger_input_title_label"))
    # Run debugger
    ui_we_want_to_set.debug_menu.run_debugger_action = QAction(
        language_wrapper.language_word_dict.get("run_menu_run_debugger"))
    ui_we_want_to_set.debug_menu.run_debugger_action.triggered.connect(
        lambda: run_debugger(ui_we_want_to_set)
    )
    ui_we_want_to_set.debug_menu.addAction(ui_we_want_to_set.debug_menu.run_debugger_action)
    # Show debugger input
    ui_we_want_to_set.debug_menu.show_shell_input = QAction(
        language_wrapper.language_word_dict.get("show_debugger_input"))
    ui_we_want_to_set.debug_menu.show_shell_input.triggered.connect(
        lambda: show_debugger_input(ui_we_want_to_set)
    )
    ui_we_want_to_set.debug_menu.addAction(ui_we_want_to_set.debug_menu.show_shell_input)


def run_debugger(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_debug_menu.py run_debugger ui_we_want_to_set: {ui_we_want_to_set}")
    widget = ui_we_want_to_set.tab_widget.currentWidget()
    if isinstance(widget, EditorWidget):
        if widget.exec_python_debugger is None:
            widget.python_compiler = ui_we_want_to_set.python_compiler
            if choose_file_get_save_file_path(ui_we_want_to_set):
                code_exec = ExecManager(widget, program_encoding=ui_we_want_to_set.encoding)
                code_exec.later_init()
                code_exec.code_result = widget.debugger_result
                code_exec.exec_code(
                    widget.current_file, exec_prefix=["-m", "pdb"]
                )
                widget.exec_python_debugger = code_exec
                widget.debugger_input = ProcessInput(widget)
                widget.debugger_input.show()
        else:
            please_close_current_running_messagebox(ui_we_want_to_set)


def show_debugger_input(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_debug_menu.py show_debugger_input ui_we_want_to_set: {ui_we_want_to_set}")
    widget = ui_we_want_to_set.tab_widget.currentWidget()
    if isinstance(widget, EditorWidget):
        widget.debugger_input = ProcessInput(widget, "debugger")
        widget.debugger_input.show()



import time
from pathlib import Path
from threading import Thread
from typing import Union

from je_editor.pyside_ui.code.plaintext_code_edit.code_edit_plaintext import CodeEditor
from je_editor.utils.file.save.save_file import write_file
from je_editor.utils.logging.loggin_instance import jeditor_logger


class CodeEditSaveThread(Thread):

    def __init__(
            self, file_to_save: Union[str, None] = None, editor: Union[None, CodeEditor] = None):
        """
        This thread is used to auto save current file.
        :param file_to_save: file we want to auto save
        :param editor: code editor to auto save
        """
        jeditor_logger.info(f"Init CodeEditSaveThread "
                            f"file_to_save: {file_to_save} "
                            f"editor: {editor}")
        super().__init__()
        self.file: str = file_to_save
        self.editor: Union[None, CodeEditor] = editor
        self.still_run: bool = True
        # set daemon
        self.daemon = True

    def run(self) -> None:
        """
        loop and save current edit file
        """
        jeditor_logger.info("CodeEditSaveThread run")
        if self.file is not None:
            path = Path(self.file)
            while path.is_file() and self.editor is not None:
                time.sleep(5)
                if self.still_run:
                    write_file(self.file, self.editor.toPlainText())
                else:
                    break



english_word_dict = {
    # Main
    "application_name": "JEditor",
    # Tab
    "tab_name_editor": "Editor",
    "tab_name_web_browser": "Web Browser",
    "tab_name_frontengine": "FrontEngine",
    "tab_name_re_edge_gpt": "Re-EdgeGPT",
    # Browser
    "browser_download_detail": "Download Detail",
    "browser_find_text": "Find Text",
    "browser_find_text_input": "Input a text to find",
    "browser_back_button": "Back",
    "browser_forward_button": "Forward",
    "browser_reload_button": "Reload",
    "browser_search_button": "Search",
    # Dialog
    "dialog_create_file": "Create File",
    "dialog_input_file_name": "Please enter right file name",
    "dialog_search_next": "Search next",
    "dialog_search_back": "Search back",
    # Editor
    "editor_code_result": "Code result",
    "editor_format_check": "Format checker",
    # Check Code Style Menu
    "check_code_style_menu_label": "Check Code Style",
    "yapf_reformat_label": "yapf",
    "black_reformat_label": "Black",
    "reformat_json_label": "Reformat JSON",
    # Dock Menu
    "dock_menu_label": "Dock",
    "dock_browser_label": "New Dock Browser",
    "dock_stackoverflow_label": "New Dock Stackoverflow",
    "dock_editor_label": "New Dock Editor",
    "dock_frontengine_label": "New Dock FrontEngine",
    "dock_ipython_label": "New Ipython(Jupyter)",
    "dock_editor_title": "Editor",
    "dock_frontengine_title": "FrontEngine",
    "dock_browser_title": "Browser",
    "dock_ipython_title": "IPython",
    # File Menu
    "file_menu_label": "File",
    "file_menu_new_file_label": "New File",
    "file_menu_open_file_label": "Open File",
    "file_menu_open_folder_label": "Open Folder",
    "file_menu_save_file_label": "Save File",
    "file_menu_encoding_label": "Encodings",
    "file_menu_font_label": "Font",
    "file_menu_font_size_label": "Font Size",
    # Help Menu
    "help_menu_label": "Help",
    "help_menu_open_github_label": "GitHub",
    "help_menu_open_doc_label": "Doc",
    "help_menu_open_about_label": "About",
    "help_menu_open_re_edge_gpt_doc_label": "Open ReEdgeGPT Doc",
    "help_menu_open_re_edge_gpt_github_label": "Open ReEdgeGPT GitHub",
    # Python Env Menu
    "python_env_menu_label": "Python Env",
    "python_env_menu_create_venv_label": "Create venv",
    "python_env_menu_pip_upgrade_label": "pip upgrade package",
    "python_env_menu_pip_label": "pip package",
    "python_env_menu_choose_interpreter_label": "Choose python interpreter",
    "python_env_menu_creating_venv_message": "Creating venv please waiting for shell exit code.",
    "python_env_menu_venv_exists": "venv already exists.",
    "python_env_menu_please_create_venv": "Please create venv first.",
    "python_env_menu_install_package_messagebox_title": "Install Package",
    "python_env_menu_install_package_messagebox_label": "What Package you want to install",
    "python_env_menu_install_or_update_package_messagebox_label": "What Package you want to install or update",
    # Run Menu
    "run_menu_label": "Run",
    "run_menu_run_program_label": "Run Program",
    "run_menu_run_debugger": "Run Debugger",
    "run_menu_run_on_shell_label": "Run On Shell",
    "run_menu_clear_result_label": "Clean Result",
    "run_menu_stop_all_program_label": "Stop All Program",
    "run_menu_stop_program_label": "Stop current program",
    "run_menu_run_help_label": "Run Help",
    "run_menu_shell_help_label": "Shell Help",
    "please_stop_current_running_shell": "Please stop current running shell first.",
    "please_stop_current_running_program": "Please stop current running program first.",
    "run_menu_run_help_tip": """
If you are unable to run a Python program, please make sure you are have python interpreter,
And if python conflict please choose another interpreter.
        """,
    "run_menu_shell_run_tip": """
When executing a shell command, if you encounter a decoding error, 
please make sure that the current encoding is consistent with the default encoding of the system shell.
        """,
    # Debugger and menu
    "editor_debugger_input_title_label": "Debugger",
    "show_debugger_input": "Show debugger input",
    # Shell and menu
    "editor_shell_input_title_label": "Shell",
    "show_shell_input": "Show shell input",
    # Program and menu
    "show_program_input": "Show program input",
    "process_input_send_command": "send command",
    # Style Menu
    "style_menu_label": "UI Style",
    # Tab Menu
    "tab_menu_label": "Tab",
    "tab_menu_add_editor_label": "Add Editor Tab",
    "tab_menu_add_frontengine_label": "Add FrontEngine Tab",
    "tab_menu_add_web_label": "Add Web Browser Tab",
    "tab_menu_add_stackoverflow_label": "Add Stackoverflow Tab",
    "tab_menu_editor_tab_name": "Editor",
    "tab_menu_frontengine_tab_name": "FrontEngine",
    "tab_menu_web_tab_name": "WEB Browser",
    "tab_menu_stackoverflow_tab_name": "Stackoverflow",
    "tab_menu_ipython_tab_name": "IPython(Jupyter)",
    "tab_menu_re_re_edge_gpt_tab_name": "ReEdgeGPT",
    # Text Menu
    "text_menu_label": "Text",
    "text_menu_label_font": "Font",
    "text_menu_label_font_size": "Font Size",
    # System tray
    "system_tray_hide": "Hide",
    "system_tray_maximized": "Maximized",
    "system_tray_normal": "Normal",
    "system_tray_close": "Close",
    # Language Menu
    "language_menu_label": "Language",
    "language_menu_bar_english": "English",
    "language_menu_bar_traditional_chinese": "Traditional Chinese",
    "language_menu_bar_please_restart_messagebox": "Please restart",
    # Qtconsole
    "please_install_qtcontsole_label": "Please python -m pip install qtconsole first.",
}



from PySide6.QtGui import QAction, Qt
from PySide6.QtWidgets import QWidget, QGridLayout, QPushButton, QInputDialog

from je_editor.pyside_ui.browser.browser_serach_lineedit import BrowserLineSearch
from je_editor.pyside_ui.browser.browser_view import BrowserView
from je_editor.utils.logging.loggin_instance import jeditor_logger
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper


class BrowserWidget(QWidget):

    def __init__(self, start_url: str = "https://www.google.com/",
                 search_prefix: str = "https://www.google.com.tw/search?q="):
        super().__init__()
        jeditor_logger.info("Init BrowserWidget "
                            f"start_url: {start_url} "
                            f"search_prefix: {search_prefix}")
        # Browser setting
        self.browser = BrowserView(start_url)
        self.search_prefix = search_prefix
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        # Top bar
        self.back_button = QPushButton(language_wrapper.language_word_dict.get("browser_back_button"))
        self.back_button.clicked.connect(self.browser.back)
        self.forward_button = QPushButton(language_wrapper.language_word_dict.get("browser_forward_button"))
        self.forward_button.clicked.connect(self.browser.forward)
        self.reload_button = QPushButton(language_wrapper.language_word_dict.get("browser_reload_button"))
        self.reload_button.clicked.connect(self.browser.reload)
        self.search_button = QPushButton(language_wrapper.language_word_dict.get("browser_search_button"))
        self.search_button.clicked.connect(self.search)
        self.url_input = BrowserLineSearch(self)
        # Action
        self.find_action = QAction()
        self.find_action.setShortcut("Ctrl+f")
        self.find_action.triggered.connect(self.find_text)
        self.addAction(self.find_action)
        # Layout
        self.grid_layout = QGridLayout()
        self.grid_layout.addWidget(self.back_button, 0, 0)
        self.grid_layout.addWidget(self.forward_button, 0, 1)
        self.grid_layout.addWidget(self.reload_button, 0, 2)
        self.grid_layout.addWidget(self.url_input, 0, 3)
        self.grid_layout.addWidget(self.search_button, 0, 4)
        self.grid_layout.addWidget(self.browser, 1, 0, -1, -1)
        self.setLayout(self.grid_layout)

    def search(self):
        jeditor_logger.info("BrowserWidget Search")
        self.browser.setUrl(f"{self.search_prefix}{self.url_input.text()}")

    def find_text(self):
        jeditor_logger.info("BrowserWidget Find Text")
        search_box = QInputDialog(self)
        search_text, press_ok = search_box.getText(
            self, language_wrapper.language_word_dict.get("browser_find_text"),
            language_wrapper.language_word_dict.get("browser_find_text_input"))
        if press_ok:
            self.browser.findText(search_text)
        else:
            self.browser.findText("")



from PySide6.QtCore import Qt
from PySide6.QtWebEngineCore import QWebEngineDownloadRequest
from PySide6.QtWidgets import QWidget, QBoxLayout, QPlainTextEdit

from je_editor.utils.logging.loggin_instance import jeditor_logger
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper


class BrowserDownloadWindow(QWidget):

    def __init__(self, download_instance: QWebEngineDownloadRequest):
        super().__init__()
        jeditor_logger.info("Init BrowserDownloadWindow "
                            f"download_instance: {download_instance}")
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        self.box_layout = QBoxLayout(QBoxLayout.Direction.TopToBottom)
        self.show_download_detail_plaintext = QPlainTextEdit()
        self.show_download_detail_plaintext.setReadOnly(True)
        self.setWindowTitle(language_wrapper.language_word_dict.get("browser_download_detail"))
        self.download_instance = download_instance
        self.download_instance.isFinishedChanged.connect(self.print_finish)
        self.download_instance.interruptReasonChanged.connect(self.print_interrupt)
        self.download_instance.stateChanged.connect(self.print_state)
        self.download_instance.accept()
        self.box_layout.addWidget(self.show_download_detail_plaintext)
        self.setLayout(self.box_layout)


    def print_finish(self):
        jeditor_logger.info("BrowserDownloadWindow Print Download is Finished")
        self.show_download_detail_plaintext.appendPlainText(str(self.download_instance.isFinished()))

    def print_interrupt(self):
        jeditor_logger.info("BrowserDownloadWindow Print interruptReason")
        self.show_download_detail_plaintext.appendPlainText(str(self.download_instance.interruptReason()))

    def print_state(self):
        jeditor_logger.info("BrowserDownloadWindow Print State")
        self.show_download_detail_plaintext.appendPlainText(str(self.download_instance.state()))



from __future__ import annotations

from typing import TYPE_CHECKING

from je_editor.utils.logging.loggin_instance import jeditor_logger
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain
    from je_editor.pyside_ui.code.shell_process.shell_exec import ShellManager
    from je_editor.pyside_ui.code.code_process.code_exec import ExecManager

import pathlib
from pathlib import Path
from typing import Union

from PySide6.QtCore import Qt, QFileInfo, QDir, QTimer
from PySide6.QtWidgets import QWidget, QGridLayout, QSplitter, QScrollArea, QFileSystemModel, QTreeView, QTabWidget

from je_editor.pyside_ui.code.auto_save.auto_save_manager import auto_save_manager_dict, init_new_auto_save_thread, \
    file_is_open_manager_dict
from je_editor.pyside_ui.code.auto_save.auto_save_thread import CodeEditSaveThread
from je_editor.pyside_ui.code.code_format.pep8_format import PEP8FormatChecker
from je_editor.pyside_ui.code.plaintext_code_edit.code_edit_plaintext import CodeEditor
from je_editor.pyside_ui.code.textedit_code_result.code_record import CodeRecord
from je_editor.pyside_ui.main_ui.save_settings.user_color_setting_file import actually_color_dict
from je_editor.pyside_ui.main_ui.save_settings.user_setting_file import user_setting_dict
from je_editor.utils.file.open.open_file import read_file


class EditorWidget(QWidget):

    def __init__(self, main_window: EditorMain):
        jeditor_logger.info(f"Init EditorWidget main_window: {main_window}")
        super().__init__()
        # Init variable
        self.checker: Union[PEP8FormatChecker, None] = None
        self.current_file = None
        self.tree_view_scroll_area = None
        self.project_treeview = None
        self.project_treeview_model = None
        self.python_compiler = None
        self.main_window = main_window
        self.tab_manager = self.main_window.tab_widget
        # Attr
        # Current execute instance if none not running
        self.exec_program: Union[None, ExecManager] = None
        self.exec_shell: Union[None, ShellManager] = None
        self.exec_python_debugger: Union[None, ExecManager] = None
        # Autosave
        self.code_save_thread: Union[CodeEditSaveThread, None] = None
        # UI
        self.grid_layout = QGridLayout(self)
        self.setWindowTitle(language_wrapper.language_word_dict.get("application_name"))
        # Treeview
        self.set_project_treeview()
        # Use to put full ui
        self.full_splitter = QSplitter()
        self.full_splitter.setOrientation(Qt.Orientation.Horizontal)
        # Code edit and result QSplitter
        self.edit_splitter = QSplitter(self.full_splitter)
        self.edit_splitter.setOrientation(Qt.Orientation.Vertical)
        # code edit and code result plaintext
        self.code_edit = CodeEditor(self)
        self.code_result = CodeRecord()
        self.code_edit_scroll_area = QScrollArea()
        self.code_edit_scroll_area.setWidgetResizable(True)
        self.code_edit_scroll_area.setViewportMargins(0, 0, 0, 0)
        self.code_edit_scroll_area.setWidget(self.code_edit)
        self.code_result_scroll_area = QScrollArea()
        self.code_result_scroll_area.setWidgetResizable(True)
        self.code_result_scroll_area.setViewportMargins(0, 0, 0, 0)
        self.code_result_scroll_area.setWidget(self.code_result)
        # Code format checker
        self.format_check_result = CodeRecord()
        self.format_check_result.setTextColor(actually_color_dict.get("warning_output_color"))
        # Debugger
        self.debugger_result = CodeRecord()
        # Code result tab
        self.code_difference_result = QTabWidget()
        self.code_difference_result.addTab(
            self.code_result_scroll_area, language_wrapper.language_word_dict.get("editor_code_result"))
        self.code_difference_result.addTab(
            self.format_check_result, language_wrapper.language_word_dict.get("editor_format_check"))
        self.code_difference_result.addTab(
            self.debugger_result, language_wrapper.language_word_dict.get("editor_debugger_input_title_label"))
        # Edit splitter
        self.edit_splitter.addWidget(self.code_edit_scroll_area)
        self.edit_splitter.addWidget(self.code_difference_result)
        self.edit_splitter.setStretchFactor(0, 3)
        self.edit_splitter.setStretchFactor(1, 1)
        self.edit_splitter.setSizes([300, 100])
        self.full_splitter.addWidget(self.project_treeview)
        self.full_splitter.addWidget(self.edit_splitter)
        self.full_splitter.setStretchFactor(0, 1)
        self.full_splitter.setStretchFactor(1, 3)
        self.full_splitter.setSizes([100, 300])
        # Font
        self.code_edit.setStyleSheet(
            f"font-size: {user_setting_dict.get('font_size', 12)}pt;"
            f"font-family: {user_setting_dict.get('font', 'Lato')};"
        )
        self.code_result.setStyleSheet(
            f"font-size: {user_setting_dict.get('font_size', 12)}pt;"
            f"font-family: {user_setting_dict.get('font', 'Lato')};"
        )
        # Add to layout
        self.grid_layout.addWidget(self.full_splitter)
        # Check format time
        self.check_format_timer = QTimer()
        self.check_format_timer.setInterval(100)
        self.check_format_timer.timeout.connect(self.check_file_format)
        self.check_format_timer.start()

    def set_project_treeview(self) -> None:
        jeditor_logger.info("EditorWidget set_project_treeview")
        self.project_treeview_model = QFileSystemModel()
        self.project_treeview_model.setRootPath(QDir.currentPath())
        self.project_treeview = QTreeView()
        self.project_treeview.setModel(self.project_treeview_model)
        if self.main_window.working_dir is None:
            self.project_treeview.setRootIndex(
                self.project_treeview_model.index(str(Path.cwd()))
            )
        else:
            self.project_treeview.setRootIndex(
                self.project_treeview_model.index(self.main_window.working_dir)
            )
        self.tree_view_scroll_area = QScrollArea()
        self.tree_view_scroll_area.setWidgetResizable(True)
        self.tree_view_scroll_area.setViewportMargins(0, 0, 0, 0)
        self.tree_view_scroll_area.setWidget(self.project_treeview)
        self.grid_layout.addWidget(self.tree_view_scroll_area, 0, 0, 0, 1)
        self.project_treeview.clicked.connect(self.treeview_click)

    def check_is_open(self, path: Path):
        jeditor_logger.info(f"EditorWidget check_is_open path: {path}")
        if file_is_open_manager_dict.get(str(path), None) is not None:
            widget: QWidget = self.tab_manager.findChild(EditorWidget, str(path))
            if widget is None:
                file_is_open_manager_dict.pop(str(path), None)
            else:
                self.tab_manager.setCurrentWidget(widget)
                return False
        else:
            file_is_open_manager_dict.update({str(path): str(path)})
            return True

    def open_an_file(self, path: Path) -> bool:
        """
        :param path: open file path
        :return: return False if file tab exists
        """
        jeditor_logger.info(f"EditorWidget open_an_file path: {path}")
        if not self.check_is_open(path):
            return False
        file, file_content = read_file(str(path))
        self.code_edit.setPlainText(
            file_content
        )
        self.current_file = file
        user_setting_dict.update({"last_file": str(self.current_file)})
        if self.current_file is not None and self.code_save_thread is None:
            init_new_auto_save_thread(self.current_file, self)
        else:
            self.code_save_thread.file = self.current_file
        self.rename_self_tab()
        return True

    def treeview_click(self) -> None:
        jeditor_logger.info("EditorWidget treeview_click")
        clicked_item: QFileSystemModel = self.project_treeview.selectedIndexes()[0]
        file_info: QFileInfo = self.project_treeview.model().fileInfo(clicked_item)
        path = pathlib.Path(file_info.absoluteFilePath())
        if path.is_file():
            self.open_an_file(path)

    def rename_self_tab(self):
        jeditor_logger.info("EditorWidget rename_self_tab")
        if self.tab_manager.currentWidget() is self:
            self.tab_manager.setTabText(
                self.tab_manager.currentIndex(), str(Path(self.current_file)))
            self.setObjectName(str(Path(self.current_file)))

    def check_file_format(self):
        jeditor_logger.info("EditorWidget check_file_format")
        if self.current_file:
            if self.checker is None:
                self.checker = PEP8FormatChecker(self.current_file)
            elif self.checker.current_file != self.current_file:
                self.checker = PEP8FormatChecker(self.current_file)
            else:
                self.checker.check_all_format()
                self.format_check_result.clear()
                for error in self.checker.error_list:
                    self.format_check_result.append(error)

    def close(self) -> bool:
        jeditor_logger.info("EditorWidget close")
        self.check_format_timer.stop()
        if self.code_save_thread is not None:
            self.code_save_thread.still_run = False
            self.code_save_thread = None
        if self.current_file:
            file_is_open_manager_dict.pop(str(Path(self.current_file)), None)
            auto_save_manager_dict.pop(self.current_file, None)
        return super().close()



from __future__ import annotations

from typing import TYPE_CHECKING

from PySide6.QtGui import QAction

from je_editor.pyside_ui.main_ui.save_settings.user_setting_file import user_setting_dict
from je_editor.utils.logging.loggin_instance import jeditor_logger

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain

from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper


def set_style_menu(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_style_menu.py set_style_menu ui_we_want_to_set: {ui_we_want_to_set}")
    ui_we_want_to_set.menu.style_menu = ui_we_want_to_set.menu.addMenu(
        language_wrapper.language_word_dict.get("style_menu_label")
    )
    for style in [
        'dark_amber.xml', 'dark_blue.xml', 'dark_cyan.xml', 'dark_lightgreen.xml', 'dark_pink.xml',
        'dark_purple.xml', 'dark_red.xml', 'dark_teal.xml', 'dark_yellow.xml', 'light_amber.xml',
        'light_blue.xml', 'light_cyan.xml', 'light_cyan_500.xml', 'light_lightgreen.xml',
        'light_pink.xml', 'light_purple.xml', 'light_red.xml', 'light_teal.xml', 'light_yellow.xml'
    ]:
        change_style_action = QAction(style, parent=ui_we_want_to_set.menu.style_menu)
        change_style_action.triggered.connect(
            lambda checked=False, action=change_style_action: set_style(ui_we_want_to_set, action))
        ui_we_want_to_set.menu.style_menu.addAction(change_style_action)


def set_style(ui_we_want_to_set: EditorMain, action: QAction) -> None:
    jeditor_logger.info("build_style_menu.py set_style "
                        f"ui_we_want_to_set: {ui_we_want_to_set} "
                        f"action: {action}")
    ui_we_want_to_set.apply_stylesheet(ui_we_want_to_set, action.text())
    user_setting_dict.update({"ui_style": action.text()})



from __future__ import annotations

from typing import TYPE_CHECKING

from je_editor.pyside_ui.main_ui.editor.editor_widget import EditorWidget
from je_editor.pyside_ui.main_ui.editor.process_input import ProcessInput
from je_editor.pyside_ui.main_ui.menu.run_menu.under_run_menu.utils import please_close_current_running_messagebox
from je_editor.utils.logging.loggin_instance import jeditor_logger

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorMain
from PySide6.QtGui import QAction

from je_editor.pyside_ui.code.shell_process.shell_exec import ShellManager
from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper


def set_shell_menu(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_shell_menu.py set_shell_menu ui_we_want_to_set: {ui_we_want_to_set}")
    ui_we_want_to_set.run_shell_menu = ui_we_want_to_set.run_menu.addMenu(
        language_wrapper.language_word_dict.get("run_menu_run_on_shell_label"))
    # Run on shell
    ui_we_want_to_set.run_shell_menu.run_on_shell_action = QAction(
        language_wrapper.language_word_dict.get("run_menu_run_on_shell_label"))
    ui_we_want_to_set.run_shell_menu.run_on_shell_action.triggered.connect(
        lambda: shell_exec(ui_we_want_to_set)
    )
    ui_we_want_to_set.run_shell_menu.addAction(ui_we_want_to_set.run_shell_menu.run_on_shell_action)
    # Show shell input
    ui_we_want_to_set.run_shell_menu.show_shell_input = QAction(
        language_wrapper.language_word_dict.get("show_shell_input"))
    ui_we_want_to_set.run_shell_menu.show_shell_input.triggered.connect(
        lambda: show_shell_input(ui_we_want_to_set)
    )
    ui_we_want_to_set.run_shell_menu.addAction(ui_we_want_to_set.run_shell_menu.show_shell_input)


def shell_exec(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_shell_menu.py shell_exec ui_we_want_to_set: {ui_we_want_to_set}")
    widget = ui_we_want_to_set.tab_widget.currentWidget()
    if isinstance(widget, EditorWidget):
        if widget.exec_shell is None:
            shell_command = ShellManager(
                main_window=widget,
                shell_encoding=ui_we_want_to_set.encoding)
            shell_command.later_init()
            shell_command.exec_shell(
                widget.code_edit.toPlainText()
            )
            widget.exec_shell = shell_command
        else:
            please_close_current_running_messagebox(ui_we_want_to_set)


def show_shell_input(ui_we_want_to_set: EditorMain) -> None:
    jeditor_logger.info(f"build_shell_menu.py show_shell_input ui_we_want_to_set: {ui_we_want_to_set}")
    widget = ui_we_want_to_set.tab_widget.currentWidget()
    if isinstance(widget, EditorWidget):
        widget.shell_input = ProcessInput(widget, "shell")
        widget.shell_input.show()



traditional_chinese_word_dict = {
    # Main
    "application_name": "JEditor",
    # Tab
    "tab_name_editor": "編輯器",
    "tab_name_web_browser": "網頁瀏覽器",
    "tab_name_frontengine": "前景引擎",
    "tab_name_re_edge_gpt": "Re-EdgeGPT",
    # Browser
    "browser_download_detail": "下載資訊",
    "browser_find_text": "搜尋文字",
    "browser_find_text_input": "輸入要尋找的文字",
    "browser_back_button": "上一頁",
    "browser_forward_button": "下一頁",
    "browser_reload_button": "重新載入",
    "browser_search_button": "搜尋",
    # Dialog
    "dialog_create_file": "建立檔案",
    "dialog_input_file_name": "請輸入正確格式的檔案名稱",
    "dialog_search_next": "搜尋下一個",
    "dialog_search_back": "搜尋上一個",
    # Editor
    "editor_code_result": "程式運行結果",
    "editor_format_check": "程式格式檢查",
    # Check Code Style Menu
    "check_code_style_menu_label": "程式碼格式檢查器",
    "yapf_reformat_label": "yapf",
    "black_reformat_label": "Black",
    "reformat_json_label": "重新格式化 JSON",
    # Dock Menu
    "dock_menu_label": "區域",
    "dock_browser_label": "新瀏覽器區域",
    "dock_stackoverflow_label": "新 Stackoverflow 區域",
    "dock_editor_label": "新編輯器區域",
    "dock_frontengine_label": "新前景引擎區域",
    "dock_ipython_label": "New Ipython(Jupyter)",
    "dock_editor_title": "編輯器",
    "dock_frontengine_title": "前景引擎",
    "dock_browser_title": "瀏覽器",
    "dock_ipython_title": "IPython(Jupyter)",
    # File Menu
    "file_menu_label": "檔案",
    "file_menu_new_file_label": "新檔案",
    "file_menu_open_file_label": "開啟檔案",
    "file_menu_open_folder_label": "開啟資料夾",
    "file_menu_save_file_label": "儲存檔案",
    "file_menu_encoding_label": "編碼",
    "file_menu_font_label": "字體",
    "file_menu_font_size_label": "字體大小",
    # Help Menu
    "help_menu_label": "幫助",
    "help_menu_open_github_label": "GitHub",
    "help_menu_open_doc_label": "文件",
    "help_menu_open_about_label": "關於",
    "help_menu_open_re_edge_gpt_doc_label": "開啟 ReEdgeGPT 文件",
    "help_menu_open_re_edge_gpt_github_label": "開啟 ReEdgeGPT GitHub",
    # Python Env Menu
    "python_env_menu_label": "Python 環境",
    "python_env_menu_create_venv_label": "建立 venv 虛擬環境",
    "python_env_menu_pip_upgrade_label": "升級與安裝套件",
    "python_env_menu_pip_label": "安裝套件",
    "python_env_menu_choose_interpreter_label": "選擇 Python 直譯器",
    "python_env_menu_creating_venv_message": "建立虛擬環境中, 請等待 Shell exit code",
    "python_env_menu_venv_exists": "虛擬環境已經存在",
    "python_env_menu_please_create_venv": "請先建立虛擬環境",
    "python_env_menu_install_package_messagebox_title": "安裝套件",
    "python_env_menu_install_package_messagebox_label": "想安裝甚麼套件?",
    "python_env_menu_install_or_update_package_messagebox_label": "想安裝或升級甚麼套件",
    # Run Menu
    "run_menu_label": "運行",
    "run_menu_run_program_label": "運行程式",
    "run_menu_run_debugger": "運行除錯器",
    "run_menu_run_on_shell_label": "在終端運行",
    "run_menu_clear_result_label": "清除結果",
    "run_menu_stop_all_program_label": "關閉所有程式",
    "run_menu_stop_program_label": "關閉當前程式",
    "run_menu_run_help_label": "運行幫助",
    "run_menu_shell_help_label": "終端運行幫助",
    "please_stop_current_running_shell": "請先關閉目前正在運行的終端",
    "please_stop_current_running_program": "請先關閉目前正在運行的程式",
    "run_menu_run_help_tip": """
如果不能運行程式, 請檢查是否有 Python 直譯器,如有 Python 衝突請選擇其他直譯器.
        """,
    "run_menu_shell_run_tip": """
如果有亂碼等問題請重新選擇編碼至與系統 Shell 編碼相同
        """,
    # Debugger
    "editor_debugger_input_title_label": "除錯器",
    "show_debugger_input": "開啟除錯輸入",
    # Shell and menu
    "editor_shell_input_title_label": "Shell",
    "show_shell_input": "開啟 Shell 輸入",
    # Program and menu
    "show_program_input": "開啟程式輸入",
    "process_input_send_command": "傳送指令",
    # Style Menu
    "style_menu_label": "UI 風格",
    # Tab Menu
    "tab_menu_label": "分頁",
    "tab_menu_add_editor_label": "新增編輯器分頁",
    "tab_menu_add_frontengine_label": "新增前景引擎分頁",
    "tab_menu_add_web_label": "新增瀏覽器分頁",
    "tab_menu_add_stackoverflow_label": "新增 Stackoverflow 分頁",
    "tab_menu_editor_tab_name": "編輯器",
    "tab_menu_frontengine_tab_name": "前景引擎",
    "tab_menu_web_tab_name": "瀏覽器",
    "tab_menu_stackoverflow_tab_name": "Stackoverflow",
    "tab_menu_ipython_tab_name": "IPython(Jupyter)",
    "tab_menu_re_re_edge_gpt_tab_name": "ReEdgeGPT",
    # Text Menu
    "text_menu_label": "文字",
    "text_menu_label_font": "字體",
    "text_menu_label_font_size": "字體大小",
    # System tray
    "system_tray_hide": "隱藏",
    "system_tray_maximized": "最大化",
    "system_tray_normal": "正常顯示",
    "system_tray_close": "關閉",
    # Language Menu
    "language_menu_label": "語言",
    "language_menu_bar_english": "英文",
    "language_menu_bar_traditional_chinese": "繁體中文",
    "language_menu_bar_please_restart_messagebox": "請重啟以應用更改",
    # Qtconsole
    "please_install_qtcontsole_label": "請先 python -m pip install qtconsole",
}



from __future__ import annotations

from typing import TYPE_CHECKING

from PySide6.QtCore import Qt
from PySide6.QtWidgets import QWidget, QLineEdit, QBoxLayout, QPushButton, QHBoxLayout

from je_editor.utils.logging.loggin_instance import jeditor_logger

if TYPE_CHECKING:
    from je_editor.pyside_ui.main_ui.main_editor import EditorWidget

from je_editor.utils.multi_language.multi_language_wrapper import language_wrapper


class ProcessInput(QWidget):

    def __init__(self, main_window: EditorWidget, process_type: str = "debugger"):
        jeditor_logger.info("Init ProcessInput "
                            f"main_window: {main_window} "
                            f"process_type: {process_type}")
        super().__init__()
        # Attr
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        # UI setting
        self.main_window = main_window
        self.box_layout = QBoxLayout(QBoxLayout.Direction.TopToBottom)
        self.command_input = QLineEdit()
        self.send_command_button = QPushButton()
        self.send_command_button.setText(language_wrapper.language_word_dict.get("process_input_send_command"))
        self.box_h_layout = QHBoxLayout()
        self.box_h_layout.addWidget(self.send_command_button)
        self.box_layout.addWidget(self.command_input)
        self.box_layout.addLayout(self.box_h_layout)
        if process_type == "program":
            self.setWindowTitle(language_wrapper.language_word_dict.get("editor_program_input_title_label"))
            self.send_command_button.clicked.connect(self.program_send_command)
        elif process_type == "shell":
            self.setWindowTitle(language_wrapper.language_word_dict.get("editor_shell_input_title_label"))
            self.send_command_button.clicked.connect(self.shell_send_command)
        else:
            self.setWindowTitle(language_wrapper.language_word_dict.get("editor_debugger_input_title_label"))
            self.send_command_button.clicked.connect(self.debugger_send_command)
            self.main_window.code_difference_result.setCurrentWidget(self.main_window.debugger_result)
        self.setLayout(self.box_layout)

    def debugger_send_command(self):
        jeditor_logger.info("EditorWidget debugger_send_command")
        if self.main_window.exec_python_debugger is not None:
            process_stdin = self.main_window.exec_python_debugger.process.stdin
            if process_stdin is not None:
                process_stdin.write(self.command_input.text().encode() + b"\n")
                process_stdin.flush()

    def shell_send_command(self):
        jeditor_logger.info("EditorWidget shell_send_command")
        if self.main_window.exec_shell is not None:
            process_stdin = self.main_window.exec_shell.process.stdin
            if process_stdin is not None:
                process_stdin.write(self.command_input.text().encode() + b"\n")
                process_stdin.flush()

    def program_send_command(self):
        jeditor_logger.info("EditorWidget program_send_command")
        if self.main_window.exec_program is not None:
            process_stdin = self.main_window.exec_program.process.stdin
            if process_stdin is not None:
                process_stdin.write(self.command_input.text().encode() + b"\n")
                process_stdin.flush()



from os import getcwd
from pathlib import Path
from typing import Dict

from PySide6.QtGui import QColor

from je_editor.pyside_ui.main_ui.save_settings.setting_utils import write_setting
from je_editor.utils.json.json_file import read_json
from je_editor.utils.logging.loggin_instance import jeditor_logger


def update_actually_color_dict():
    jeditor_logger.info(f"user_color_setting_file.py update_actually_color_dict")
    actually_color_dict.update(
        {
            "line_number_color": QColor(
                user_setting_color_dict.get("line_number_color")[0],
                user_setting_color_dict.get("line_number_color")[1],
                user_setting_color_dict.get("line_number_color")[2],
            ),
            "line_number_background_color": QColor(
                user_setting_color_dict.get("line_number_background_color")[0],
                user_setting_color_dict.get("line_number_background_color")[1],
                user_setting_color_dict.get("line_number_background_color")[2],
            ),
            "current_line_color": QColor(
                user_setting_color_dict.get("current_line_color")[0],
                user_setting_color_dict.get("current_line_color")[1],
                user_setting_color_dict.get("current_line_color")[2],
            ),
            "normal_output_color": QColor(
                user_setting_color_dict.get("normal_output_color")[0],
                user_setting_color_dict.get("normal_output_color")[1],
                user_setting_color_dict.get("normal_output_color")[2],
            ),
            "error_output_color": QColor(
                user_setting_color_dict.get("error_output_color")[0],
                user_setting_color_dict.get("error_output_color")[1],
                user_setting_color_dict.get("error_output_color")[2],
            ),
            "warning_output_color": QColor(
                user_setting_color_dict.get("warning_output_color")[0],
                user_setting_color_dict.get("warning_output_color")[1],
                user_setting_color_dict.get("warning_output_color")[2],
            )
        }
    )


user_setting_color_dict: Dict[str, list] = {
    "line_number_color": [255, 255, 255],
    "line_number_background_color": [179, 179, 179],
    "current_line_color": [148, 148, 184],
    "normal_output_color": [255, 255, 255],
    "error_output_color": [255, 0, 0],
    "warning_output_color": [204, 204, 0]
}

actually_color_dict: Dict[str, QColor] = {
}
update_actually_color_dict()


def write_user_color_setting() -> None:
    jeditor_logger.info("user_color_setting_file.py write_user_color_setting")
    write_setting(user_setting_color_dict, "user_color_setting.json")


def read_user_color_setting() -> None:
    jeditor_logger.info("user_color_setting_file.py read_user_color_setting")
    user_color_setting_file = Path(getcwd() + "/.jeditor/user_color_setting.json")
    if user_color_setting_file.exists() and user_color_setting_file.is_file():
        user_setting_color_dict.update(read_json(str(user_color_setting_file)))
